@@ -1,8 +1,11 @@
 package main
 
 import (
+	"bytes"
 	"encoding/json"
 	"fmt"
+	"io"
+	"io/ioutil"
 	"os"
 	"strconv"
 	"strings"
@@ -12,20 +15,21 @@ import (
 
 	"github.com/lxc/lxd"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/gnuflag"
 	"github.com/lxc/lxd/shared/i18n"
 	"github.com/lxc/lxd/shared/termios"
 )
 
-type envFlag []string
+type envList []string
 
-func (f *envFlag) String() string {
+func (f *envList) String() string {
 	return fmt.Sprint(*f)
 }
 
-func (f *envFlag) Set(value string) error {
+func (f *envList) Set(value string) error {
 	if f == nil {
-		*f = make(envFlag, 1)
+		*f = make(envList, 1)
 	} else {
 		*f = append(*f, value)
 	}
@@ -33,8 +37,11 @@ func (f *envFlag) Set(value string) error {
 }
 
 type execCmd struct {
-	modeFlag string
-	envArgs  envFlag
+	modeFlag            string
+	envArgs             envList
+	forceInteractive    bool
+	forceNonInteractive bool
+	disableStdin        bool
 }
 
 func (c *execCmd) showByDefault() bool {
@@ -43,9 +50,9 @@ func (c *execCmd) showByDefault() bool {
 
 func (c *execCmd) usage() string {
 	return i18n.G(
-		`Execute the specified command in a container.
+		`Usage: lxc exec [<remote>:]<container> [-t] [-T] [-n] [--mode=auto|interactive|non-interactive] [--env KEY=VALUE...] [--] <command line>
 
-lxc exec [<remote>:]<container> [--mode=auto|interactive|non-interactive] [--env KEY=VALUE...] [--] <command line>
+Execute commands in containers.
 
 Mode defaults to non-interactive, interactive mode is selected if both stdin AND stdout are terminals (stderr is ignored).`)
 }
@@ -53,6 +60,9 @@ Mode defaults to non-interactive, interactive mode is selected if both stdin AND
 func (c *execCmd) flags() {
 	gnuflag.Var(&c.envArgs, "env", i18n.G("Environment variable to set (e.g. HOME=/home/foo)"))
 	gnuflag.StringVar(&c.modeFlag, "mode", "auto", i18n.G("Override the terminal mode (auto, interactive or non-interactive)"))
+	gnuflag.BoolVar(&c.forceInteractive, "t", false, i18n.G("Force pseudo-terminal allocation"))
+	gnuflag.BoolVar(&c.forceNonInteractive, "T", false, i18n.G("Disable pseudo-terminal allocation"))
+	gnuflag.BoolVar(&c.disableStdin, "n", false, i18n.G("Disable stdin (reads from /dev/null)"))
 }
 
 func (c *execCmd) sendTermSize(control *websocket.Conn) error {
@@ -68,7 +78,7 @@ func (c *execCmd) sendTermSize(control *websocket.Conn) error {
 		return err
 	}
 
-	msg := shared.ContainerExecControl{}
+	msg := api.ContainerExecControl{}
 	msg.Command = "window-resize"
 	msg.Args = make(map[string]string)
 	msg.Args["width"] = strconv.Itoa(width)
@@ -92,9 +102,9 @@ func (c *execCmd) forwardSignal(control *websocket.Conn, sig syscall.Signal) err
 		return err
 	}
 
-	msg := shared.ContainerExecControl{}
+	msg := api.ContainerExecControl{}
 	msg.Command = "signal"
-	msg.Signal = sig
+	msg.Signal = int(sig)
 
 	buf, err := json.Marshal(msg)
 	if err != nil {
@@ -111,12 +121,23 @@ func (c *execCmd) run(config *lxd.Config, args []string) error {
 		return errArgs
 	}
 
+	if c.forceInteractive && c.forceNonInteractive {
+		return fmt.Errorf(i18n.G("You can't pass -t and -T at the same time"))
+	}
+
+	if c.modeFlag != "auto" && (c.forceInteractive || c.forceNonInteractive) {
+		return fmt.Errorf(i18n.G("You can't pass -t or -T at the same time as --mode"))
+	}
+
 	remote, name := config.ParseRemoteAndContainer(args[0])
 	d, err := lxd.NewClient(config, remote)
 	if err != nil {
 		return err
 	}
 
+	/* FIXME: Default values for HOME and USER are now handled by LXD.
+	   This code should be removed after most users upgraded.
+	*/
 	env := map[string]string{"HOME": "/root", "USER": "root"}
 	if myTerm, ok := c.getTERM(); ok {
 		env["TERM"] = myTerm
@@ -134,9 +155,11 @@ func (c *execCmd) run(config *lxd.Config, args []string) error {
 	cfd := int(syscall.Stdin)
 
 	var interactive bool
-	if c.modeFlag == "interactive" {
+	if c.disableStdin {
+		interactive = false
+	} else if c.modeFlag == "interactive" || c.forceInteractive {
 		interactive = true
-	} else if c.modeFlag == "non-interactive" {
+	} else if c.modeFlag == "non-interactive" || c.forceNonInteractive {
 		interactive = false
 	} else {
 		interactive = termios.IsTerminal(cfd) && termios.IsTerminal(int(syscall.Stdout))
@@ -164,8 +187,14 @@ func (c *execCmd) run(config *lxd.Config, args []string) error {
 		}
 	}
 
+	var stdin io.ReadCloser
+	stdin = os.Stdin
+	if c.disableStdin {
+		stdin = ioutil.NopCloser(bytes.NewReader(nil))
+	}
+
 	stdout := c.getStdout()
-	ret, err := d.Exec(name, args[1:], env, os.Stdin, stdout, os.Stderr, handler, width, height)
+	ret, err := d.Exec(name, args[1:], env, stdin, stdout, os.Stderr, handler, width, height)
 	if err != nil {
 		return err
 	}
