@@ -6,6 +6,7 @@ import (
 
 	"github.com/lxc/lxd"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/i18n"
 	"github.com/lxc/lxd/shared/version"
 )
@@ -20,16 +21,14 @@ func (c *launchCmd) showByDefault() bool {
 
 func (c *launchCmd) usage() string {
 	return i18n.G(
-		`Launch a container from a particular image.
+		`Usage: lxc launch [<remote>:]<image> [<remote>:][<name>] [--ephemeral|-e] [--profile|-p <profile>...] [--config|-c <key=value>...] [--network|-n <network>] [--storage|-s <pool>]
 
-lxc launch [<remote>:]<image> [<remote>:][<name>] [--ephemeral|-e] [--profile|-p <profile>...] [--config|-c <key=value>...] [--network|-n <network>]
-
-Launches a container using the specified image and name.
+Create and start containers from images.
 
 Not specifying -p will result in the default profile.
 Specifying "-p" with no argument will result in no profile.
 
-Example:
+Examples:
     lxc launch ubuntu:16.04 u1`)
 }
 
@@ -62,7 +61,7 @@ func (c *launchCmd) run(config *lxd.Config, args []string) error {
 	 * initRequestedEmptyProfiles means user requested empty
 	 * !initRequestedEmptyProfiles but len(profArgs) == 0 means use profile default
 	 */
-	var resp *lxd.Response
+	var resp *api.Response
 	profiles := []string{}
 	for _, p := range c.init.profArgs {
 		profiles = append(profiles, p)
@@ -70,7 +69,7 @@ func (c *launchCmd) run(config *lxd.Config, args []string) error {
 
 	iremote, image = c.init.guessImage(config, d, remote, iremote, image)
 
-	devicesMap := map[string]shared.Device{}
+	devicesMap := map[string]map[string]string{}
 	if c.init.network != "" {
 		network, err := d.NetworkGet(c.init.network)
 		if err != nil {
@@ -78,9 +77,22 @@ func (c *launchCmd) run(config *lxd.Config, args []string) error {
 		}
 
 		if network.Type == "bridge" {
-			devicesMap[c.init.network] = shared.Device{"type": "nic", "nictype": "bridged", "parent": c.init.network}
+			devicesMap[c.init.network] = map[string]string{"type": "nic", "nictype": "bridged", "parent": c.init.network}
 		} else {
-			devicesMap[c.init.network] = shared.Device{"type": "nic", "nictype": "macvlan", "parent": c.init.network}
+			devicesMap[c.init.network] = map[string]string{"type": "nic", "nictype": "macvlan", "parent": c.init.network}
+		}
+	}
+
+	// Check if the specified storage pool exists.
+	if c.init.storagePool != "" {
+		_, err := d.StoragePoolGet(c.init.storagePool)
+		if err != nil {
+			return err
+		}
+		devicesMap["root"] = map[string]string{
+			"type": "disk",
+			"path": "/",
+			"pool": c.init.storagePool,
 		}
 	}
 
@@ -89,7 +101,6 @@ func (c *launchCmd) run(config *lxd.Config, args []string) error {
 	} else {
 		resp, err = d.Init(name, iremote, image, &profiles, configMap, devicesMap, c.init.ephem)
 	}
-
 	if err != nil {
 		return err
 	}
@@ -100,11 +111,13 @@ func (c *launchCmd) run(config *lxd.Config, args []string) error {
 	if name == "" {
 		op, err := resp.MetadataAsOperation()
 		if err != nil {
+			progress.Done("")
 			return fmt.Errorf(i18n.G("didn't get any affected image, container or snapshot from server"))
 		}
 
 		containers, ok := op.Resources["containers"]
 		if !ok || len(containers) == 0 {
+			progress.Done("")
 			return fmt.Errorf(i18n.G("didn't get any affected image, container or snapshot from server"))
 		}
 
@@ -112,20 +125,24 @@ func (c *launchCmd) run(config *lxd.Config, args []string) error {
 		toScan := strings.Replace(containers[0], "/", " ", -1)
 		count, err := fmt.Sscanf(toScan, " %s containers %s", &restVersion, &name)
 		if err != nil {
+			progress.Done("")
 			return err
 		}
 
 		if count != 2 {
+			progress.Done("")
 			return fmt.Errorf(i18n.G("bad number of things scanned from image, container or snapshot"))
 		}
 
 		if restVersion != version.APIVersion {
+			progress.Done("")
 			return fmt.Errorf(i18n.G("got bad version"))
 		}
 	}
 	fmt.Printf(i18n.G("Creating %s")+"\n", name)
 
 	if err = d.WaitForSuccess(resp.Operation); err != nil {
+		progress.Done("")
 		return err
 	}
 	progress.Done("")
