@@ -12,6 +12,7 @@ import (
 
 	"github.com/lxc/lxd"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/i18n"
 	"github.com/lxc/lxd/shared/termios"
 )
@@ -46,50 +47,89 @@ func (c *profileCmd) profileEditHelp() string {
 
 func (c *profileCmd) usage() string {
 	return i18n.G(
-		`Manage configuration profiles.
+		`Usage: lxc profile <subcommand> [options]
+
+Manage container configuration profiles.
+
+*Profile configuration*
+lxc profile list [<remote>:]
+    List available profiles.
+
+lxc profile show [<remote>:]<profile>
+    Show details of a profile.
+
+lxc profile create [<remote>:]<profile>
+    Create a profile.
+
+lxc profile copy [<remote>:]<profile> [<remote>:]<profile>
+    Copy the profile.
+
+lxc profile get [<remote>:]<profile> <key>
+    Get profile configuration.
+
+lxc profile set [<remote>:]<profile> <key> <value>
+    Set profile configuration.
+
+lxc profile unset [<remote>:]<profile> <key>
+    Unset profile configuration.
+
+lxc profile delete [<remote>:]<profile>
+    Delete a profile.
 
-lxc profile list [<remote>:]                                    List available profiles.
-lxc profile show [<remote>:]<profile>                           Show details of a profile.
-lxc profile create [<remote>:]<profile>                         Create a profile.
-lxc profile copy [<remote>:]<profile> [<remote>:]<profile>      Copy the profile.
-lxc profile get [<remote>:]<profile> <key>                      Get profile configuration.
-lxc profile set [<remote>:]<profile> <key> <value>              Set profile configuration.
-lxc profile unset [<remote>:]<profile> <key>                    Unset profile configuration.
-lxc profile delete [<remote>:]<profile>                         Delete a profile.
 lxc profile edit [<remote>:]<profile>
     Edit profile, either by launching external editor or reading STDIN.
-    Example: lxc profile edit <profile> # launch editor
-             cat profile.yaml | lxc profile edit <profile> # read from profile.yaml
 
+*Profile assignment*
 lxc profile assign [<remote>:]<container> <profiles>
-    Assign a comma-separated list of profiles to a container, in order.
-    All profiles passed in this call (and only those) will be applied
-    to the specified container, i.e. it sets the list of profiles exactly to
-    those specified in this command. To add/remove a particular profile from a
-    container, use {add|remove} below.
-    Example: lxc profile assign foo default,bar # Apply default and bar
-             lxc profile assign foo default # Only default is active
-             lxc profile assign '' # no profiles are applied anymore
-             lxc profile assign bar,default # Apply default second now
-lxc profile add [<remote>:]<container> <profile> # add a profile to a container
-lxc profile remove [<remote>:]<container> <profile> # remove the profile from a container
+    Replace the current set of profiles for the container by the one provided.
+
+lxc profile add [<remote>:]<container> <profile>
+    Add a profile to a container
+
+lxc profile remove [<remote>:]<container> <profile>
+    Remove the profile from a container
+
+*Device management*
+lxc profile device list [<remote>:]<profile>
+    List devices in the given profile.
+
+lxc profile device show [<remote>:]<profile>
+    Show full device details in the given profile.
+
+lxc profile device remove [<remote>:]<profile> <name>
+    Remove a device from a profile.
+
+lxc profile device get [<remote>:]<profile> <name> <key>
+    Get a device property.
+
+lxc profile device set [<remote>:]<profile> <name> <key> <value>
+    Set a device property.
+
+lxc profile device unset [<remote>:]<profile> <name> <key>
+    Unset a device property.
 
-Devices:
-lxc profile device list [<remote>:]<profile>                                List devices in the given profile.
-lxc profile device show [<remote>:]<profile>                                Show full device details in the given profile.
-lxc profile device remove [<remote>:]<profile> <name>                       Remove a device from a profile.
-lxc profile device get [<remote>:]<profile> <name> <key>                    Get a device property.
-lxc profile device set [<remote>:]<profile> <name> <key> <value>            Set a device property.
-lxc profile device unset [<remote>:]<profile> <name> <key>                  Unset a device property.
 lxc profile device add [<remote>:]<profile> <device> <type> [key=value...]
-    Add a profile device, such as a disk or a nic, to the containers using the specified profile.`)
+    Add a profile device, such as a disk or a nic, to the containers using the specified profile.
+
+*Examples*
+cat profile.yaml | lxc profile edit <profile>
+    Update a profile using the content of profile.yaml
+
+lxc profile assign foo default,bar
+    Set the profiles for "foo" to "default" and "bar".
+
+lxc profile assign foo default
+    Reset "foo" to only using the "default" profile.
+
+lxc profile assign foo ''
+    Remove all profile from "foo"`)
 }
 
 func (c *profileCmd) flags() {}
 
 func (c *profileCmd) run(config *lxd.Config, args []string) error {
 	if len(args) < 1 {
-		return errArgs
+		return errUsage
 	}
 
 	if args[0] == "list" {
@@ -153,7 +193,7 @@ func (c *profileCmd) run(config *lxd.Config, args []string) error {
 	case "set":
 		return c.doProfileSet(client, profile, args[2:])
 	case "unset":
-		return c.doProfileSet(client, profile, args[2:])
+		return c.doProfileUnset(client, profile, args[2:])
 	case "copy":
 		return c.doProfileCopy(config, client, profile, args[2:])
 	case "show":
@@ -179,7 +219,7 @@ func (c *profileCmd) doProfileEdit(client *lxd.Client, p string) error {
 			return err
 		}
 
-		newdata := shared.ProfileConfig{}
+		newdata := api.ProfilePut{}
 		err = yaml.Unmarshal(contents, &newdata)
 		if err != nil {
 			return err
@@ -206,7 +246,7 @@ func (c *profileCmd) doProfileEdit(client *lxd.Client, p string) error {
 
 	for {
 		// Parse the text received from the editor
-		newdata := shared.ProfileConfig{}
+		newdata := api.ProfilePut{}
 		err = yaml.Unmarshal(content, &newdata)
 		if err == nil {
 			err = client.PutProfile(p, newdata)
@@ -266,7 +306,7 @@ func (c *profileCmd) doProfileAdd(client *lxd.Client, d string, p string) error
 
 	ct.Profiles = append(ct.Profiles, p)
 
-	err = client.UpdateContainerConfig(d, ct.Brief())
+	err = client.UpdateContainerConfig(d, ct.Writable())
 	if err != nil {
 		return err
 	}
@@ -297,7 +337,7 @@ func (c *profileCmd) doProfileRemove(client *lxd.Client, d string, p string) err
 
 	ct.Profiles = profiles
 
-	err = client.UpdateContainerConfig(d, ct.Brief())
+	err = client.UpdateContainerConfig(d, ct.Writable())
 	if err != nil {
 		return err
 	}
@@ -314,6 +354,10 @@ func (c *profileCmd) doProfileShow(client *lxd.Client, p string) error {
 	}
 
 	data, err := yaml.Marshal(&profile)
+	if err != nil {
+		return err
+	}
+
 	fmt.Printf("%s", data)
 
 	return nil
@@ -410,6 +454,15 @@ func (c *profileCmd) doProfileSet(client *lxd.Client, p string, args []string) e
 	return err
 }
 
+func (c *profileCmd) doProfileUnset(client *lxd.Client, p string, args []string) error {
+	// we shifted @args so so it should read "<key> [<value>]"
+	if len(args) != 1 {
+		return errArgs
+	}
+
+	return c.doProfileSet(client, p, args)
+}
+
 func (c *profileCmd) doProfileList(config *lxd.Config, args []string) error {
 	var remote string
 	if len(args) > 1 {
