@@ -35,16 +35,30 @@ func (c *remoteCmd) showByDefault() bool {
 
 func (c *remoteCmd) usage() string {
 	return i18n.G(
-		`Manage remote LXD servers.
+		`Usage: lxc remote <subcommand> [options]
 
-lxc remote add [<remote>] <IP|FQDN|URL> [--accept-certificate] [--password=PASSWORD]
-                                        [--public] [--protocol=PROTOCOL]    Add the remote <name> at <url>.
-lxc remote remove <remote>                                                  Remove the remote <name>.
-lxc remote list                                                             List all remotes.
-lxc remote rename <old name> <new name>                                     Rename remote <old> to <new>.
-lxc remote set-url <remote> <url>                                           Update <name>'s url to <url>.
-lxc remote set-default <remote>                                             Set the default remote.
-lxc remote get-default                                                      Print the default remote.`)
+Manage the list of remote LXD servers.
+
+lxc remote add [<remote>] <IP|FQDN|URL> [--accept-certificate] [--password=PASSWORD] [--public] [--protocol=PROTOCOL]
+    Add the remote <remote> at <url>.
+
+lxc remote remove <remote>
+    Remove the remote <remote>.
+
+lxc remote list
+    List all remotes.
+
+lxc remote rename <old name> <new name>
+    Rename remote <old name> to <new name>.
+
+lxc remote set-url <remote> <url>
+    Update <remote>'s url to <url>.
+
+lxc remote set-default <remote>
+    Set the default remote.
+
+lxc remote get-default
+    Print the default remote.`)
 }
 
 func (c *remoteCmd) flags() {
@@ -54,7 +68,7 @@ func (c *remoteCmd) flags() {
 	gnuflag.BoolVar(&c.public, "public", false, i18n.G("Public image server"))
 }
 
-func generateClientCertificate(config *lxd.Config) error {
+func (c *remoteCmd) generateClientCertificate(config *lxd.Config) error {
 	// Generate a client certificate if necessary.  The default repositories are
 	// either local or public, neither of which requires a client certificate.
 	// Generation of the cert is delayed to avoid unnecessary overhead, e.g in
@@ -69,7 +83,7 @@ func generateClientCertificate(config *lxd.Config) error {
 	return nil
 }
 
-func getRemoteCertificate(address string) (*x509.Certificate, error) {
+func (c *remoteCmd) getRemoteCertificate(address string) (*x509.Certificate, error) {
 	// Setup a permissive TLS config
 	tlsConfig, err := shared.GetTLSConfig("", "", "", nil)
 	if err != nil {
@@ -179,7 +193,7 @@ func (c *remoteCmd) addServer(config *lxd.Config, server string, addr string, ac
 	// HTTPS server then we need to ensure we have a client certificate before
 	// adding the remote server.
 	if rScheme != "unix" && !public {
-		err = generateClientCertificate(config)
+		err = c.generateClientCertificate(config)
 		if err != nil {
 			return err
 		}
@@ -204,7 +218,7 @@ func (c *remoteCmd) addServer(config *lxd.Config, server string, addr string, ac
 	_, err = d.GetServerConfig()
 	if err != nil {
 		// Failed to connect using the system CA, so retrieve the remote certificate
-		certificate, err = getRemoteCertificate(addr)
+		certificate, err = c.getRemoteCertificate(addr)
 		if err != nil {
 			return err
 		}
@@ -300,7 +314,7 @@ func (c *remoteCmd) removeCertificate(config *lxd.Config, remote string) {
 
 func (c *remoteCmd) run(config *lxd.Config, args []string) error {
 	if len(args) < 1 {
-		return errArgs
+		return errUsage
 	}
 
 	switch args[0] {
