@@ -3,9 +3,8 @@ package main
 import (
 	"fmt"
 
-	"github.com/lxc/lxd/lxc/config"
+	"github.com/lxc/lxd"
 	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/gnuflag"
 	"github.com/lxc/lxd/shared/i18n"
 )
@@ -38,19 +37,15 @@ func (c *restoreCmd) flags() {
 	gnuflag.BoolVar(&c.stateful, "stateful", false, i18n.G("Whether or not to restore the container's running state from snapshot (if available)"))
 }
 
-func (c *restoreCmd) run(conf *config.Config, args []string) error {
+func (c *restoreCmd) run(config *lxd.Config, args []string) error {
 	if len(args) < 2 {
 		return errArgs
 	}
 
 	var snapname = args[1]
 
-	remote, name, err := conf.ParseRemote(args[0])
-	if err != nil {
-		return err
-	}
-
-	d, err := conf.GetContainerServer(remote)
+	remote, name := config.ParseRemoteAndContainer(args[0])
+	d, err := lxd.NewClient(config, remote)
 	if err != nil {
 		return err
 	}
@@ -59,15 +54,10 @@ func (c *restoreCmd) run(conf *config.Config, args []string) error {
 		snapname = fmt.Sprintf("%s/%s", name, snapname)
 	}
 
-	req := api.ContainerPut{
-		Restore:  snapname,
-		Stateful: c.stateful,
-	}
-
-	op, err := d.UpdateContainer(name, req, "")
+	resp, err := d.RestoreSnapshot(name, snapname, c.stateful)
 	if err != nil {
 		return err
 	}
 
-	return op.Wait()
+	return d.WaitForSuccess(resp.Operation)
 }
