@@ -3,9 +3,8 @@ package main
 import (
 	"fmt"
 
-	"github.com/lxc/lxd/lxc/config"
+	"github.com/lxc/lxd"
 	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/gnuflag"
 	"github.com/lxc/lxd/shared/i18n"
 )
@@ -36,7 +35,7 @@ func (c *snapshotCmd) flags() {
 	gnuflag.BoolVar(&c.stateful, "stateful", false, i18n.G("Whether or not to snapshot the container's running state"))
 }
 
-func (c *snapshotCmd) run(conf *config.Config, args []string) error {
+func (c *snapshotCmd) run(config *lxd.Config, args []string) error {
 	if len(args) < 1 {
 		return errArgs
 	}
@@ -48,30 +47,21 @@ func (c *snapshotCmd) run(conf *config.Config, args []string) error {
 		snapname = args[1]
 	}
 
+	remote, name := config.ParseRemoteAndContainer(args[0])
+	d, err := lxd.NewClient(config, remote)
+	if err != nil {
+		return err
+	}
+
 	// we don't allow '/' in snapshot names
 	if shared.IsSnapshot(snapname) {
 		return fmt.Errorf(i18n.G("'/' not allowed in snapshot name"))
 	}
 
-	remote, name, err := conf.ParseRemote(args[0])
+	resp, err := d.Snapshot(name, snapname, c.stateful)
 	if err != nil {
 		return err
 	}
 
-	d, err := conf.GetContainerServer(remote)
-	if err != nil {
-		return err
-	}
-
-	req := api.ContainerSnapshotsPost{
-		Name:     snapname,
-		Stateful: c.stateful,
-	}
-
-	op, err := d.CreateContainerSnapshot(name, req)
-	if err != nil {
-		return err
-	}
-
-	return op.Wait()
+	return d.WaitForSuccess(resp.Operation)
 }
