@@ -3,7 +3,7 @@ package main
 import (
 	"fmt"
 
-	"github.com/lxc/lxd/lxc/config"
+	"github.com/lxc/lxd"
 	"github.com/lxc/lxd/shared/i18n"
 	"github.com/lxc/lxd/shared/version"
 )
@@ -11,7 +11,7 @@ import (
 type versionCmd struct{}
 
 func (c *versionCmd) showByDefault() bool {
-	return true
+	return false
 }
 
 func (c *versionCmd) usage() string {
@@ -24,7 +24,7 @@ Print the version number of this client tool.`)
 func (c *versionCmd) flags() {
 }
 
-func (c *versionCmd) run(conf *config.Config, args []string) error {
+func (c *versionCmd) run(_ *lxd.Config, args []string) error {
 	if len(args) > 0 {
 		return errArgs
 	}
