@@ -11,6 +11,7 @@ import (
 	"gopkg.in/lxc/go-lxc.v2"
 
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/osarch"
 	"github.com/lxc/lxd/shared/version"
 )
@@ -43,10 +44,15 @@ var api10 = []Command{
 	certificateFingerprintCmd,
 	profilesCmd,
 	profileCmd,
+	storagePoolsCmd,
+	storagePoolCmd,
+	storagePoolVolumesCmd,
+	storagePoolVolumesTypeCmd,
+	storagePoolVolumeTypeCmd,
 }
 
 func api10Get(d *Daemon, r *http.Request) Response {
-	body := shared.Jmap{
+	srv := api.ServerUntrusted{
 		/* List of API extensions in the order they were added.
 		 *
 		 * The following kind of changes require an addition to api_extensions:
@@ -56,7 +62,7 @@ func api10Get(d *Daemon, r *http.Request) Response {
 		 *  - New argument inside an existing REST API call
 		 *  - New HTTPs authentication mechanisms or protocols
 		 */
-		"api_extensions": []string{
+		APIExtensions: []string{
 			"storage_zfs_remove_snapshots",
 			"container_host_shutdown_timeout",
 			"container_syscall_filtering",
@@ -83,99 +89,126 @@ func api10Get(d *Daemon, r *http.Request) Response {
 			"id_map",
 			"network_firewall_filtering",
 			"network_routes",
+			"storage",
+			"file_delete",
+			"file_append",
+			"network_dhcp_expiry",
+			"storage_lvm_vg_rename",
+			"storage_lvm_thinpool_rename",
+			"network_vlan",
+			"image_create_aliases",
+			"container_stateless_copy",
 		},
-
-		"api_status":  "stable",
-		"api_version": version.APIVersion,
+		APIStatus:  "stable",
+		APIVersion: version.APIVersion,
+		Public:     false,
+		Auth:       "untrusted",
 	}
 
-	if d.isTrustedClient(r) {
-		body["auth"] = "trusted"
+	// If untrusted, return now
+	if !d.isTrustedClient(r) {
+		return SyncResponseETag(true, srv, nil)
+	}
 
-		/*
-		 * Based on: https://groups.google.com/forum/#!topic/golang-nuts/Jel8Bb-YwX8
-		 * there is really no better way to do this, which is
-		 * unfortunate. Also, we ditch the more accepted CharsToString
-		 * version in that thread, since it doesn't seem as portable,
-		 * viz. github issue #206.
-		 */
-		uname := syscall.Utsname{}
-		if err := syscall.Uname(&uname); err != nil {
-			return InternalError(err)
+	srv.Auth = "trusted"
+
+	/*
+	 * Based on: https://groups.google.com/forum/#!topic/golang-nuts/Jel8Bb-YwX8
+	 * there is really no better way to do this, which is
+	 * unfortunate. Also, we ditch the more accepted CharsToString
+	 * version in that thread, since it doesn't seem as portable,
+	 * viz. github issue #206.
+	 */
+	uname := syscall.Utsname{}
+	if err := syscall.Uname(&uname); err != nil {
+		return InternalError(err)
+	}
+
+	kernel := ""
+	for _, c := range uname.Sysname {
+		if c == 0 {
+			break
 		}
+		kernel += string(byte(c))
+	}
 
-		kernel := ""
-		for _, c := range uname.Sysname {
-			if c == 0 {
-				break
-			}
-			kernel += string(byte(c))
+	kernelVersion := ""
+	for _, c := range uname.Release {
+		if c == 0 {
+			break
 		}
+		kernelVersion += string(byte(c))
+	}
 
-		kernelVersion := ""
-		for _, c := range uname.Release {
-			if c == 0 {
-				break
-			}
-			kernelVersion += string(byte(c))
+	kernelArchitecture := ""
+	for _, c := range uname.Machine {
+		if c == 0 {
+			break
 		}
+		kernelArchitecture += string(byte(c))
+	}
 
-		kernelArchitecture := ""
-		for _, c := range uname.Machine {
-			if c == 0 {
-				break
-			}
-			kernelArchitecture += string(byte(c))
-		}
+	addresses, err := d.ListenAddresses()
+	if err != nil {
+		return InternalError(err)
+	}
 
-		addresses, err := d.ListenAddresses()
+	var certificate string
+	if len(d.tlsConfig.Certificates) != 0 {
+		certificate = string(pem.EncodeToMemory(&pem.Block{Type: "CERTIFICATE", Bytes: d.tlsConfig.Certificates[0].Certificate[0]}))
+	}
+
+	architectures := []string{}
+
+	for _, architecture := range d.architectures {
+		architectureName, err := osarch.ArchitectureName(architecture)
 		if err != nil {
 			return InternalError(err)
 		}
-
-		var certificate string
-		if len(d.tlsConfig.Certificates) != 0 {
-			certificate = string(pem.EncodeToMemory(&pem.Block{Type: "CERTIFICATE", Bytes: d.tlsConfig.Certificates[0].Certificate[0]}))
-		}
-
-		architectures := []string{}
-
-		for _, architecture := range d.architectures {
-			architectureName, err := osarch.ArchitectureName(architecture)
-			if err != nil {
-				return InternalError(err)
-			}
-			architectures = append(architectures, architectureName)
-		}
-
-		env := shared.Jmap{
-			"addresses":           addresses,
-			"architectures":       architectures,
-			"certificate":         certificate,
-			"driver":              "lxc",
-			"driver_version":      lxc.Version(),
-			"kernel":              kernel,
-			"kernel_architecture": kernelArchitecture,
-			"kernel_version":      kernelVersion,
-			"storage":             d.Storage.GetStorageTypeName(),
-			"storage_version":     d.Storage.GetStorageTypeVersion(),
-			"server":              "lxd",
-			"server_pid":          os.Getpid(),
-			"server_version":      version.Version}
-
-		body["environment"] = env
-		body["public"] = false
-		body["config"] = daemonConfigRender()
-	} else {
-		body["auth"] = "untrusted"
-		body["public"] = false
+		architectures = append(architectures, architectureName)
 	}
 
-	return SyncResponseETag(true, body, body["config"])
-}
+	env := api.ServerEnvironment{
+		Addresses:          addresses,
+		Architectures:      architectures,
+		Certificate:        certificate,
+		Driver:             "lxc",
+		DriverVersion:      lxc.Version(),
+		Kernel:             kernel,
+		KernelArchitecture: kernelArchitecture,
+		KernelVersion:      kernelVersion,
+		Server:             "lxd",
+		ServerPid:          os.Getpid(),
+		ServerVersion:      version.Version}
 
-type apiPut struct {
-	Config shared.Jmap `json:"config"`
+	drivers := readStoragePoolDriversCache()
+	for _, driver := range drivers {
+		// Initialize a core storage interface for the given driver.
+		sCore, err := storageCoreInit(driver)
+		if err != nil {
+			continue
+		}
+
+		if env.Storage != "" {
+			env.Storage = env.Storage + " | " + driver
+		} else {
+			env.Storage = driver
+		}
+
+		// Get the version of the storage drivers in use.
+		sVersion := sCore.GetStorageTypeVersion()
+		if env.StorageVersion != "" {
+			env.StorageVersion = env.StorageVersion + " | " + sVersion
+		} else {
+			env.StorageVersion = sVersion
+		}
+	}
+
+	fullSrv := api.Server{ServerUntrusted: srv}
+	fullSrv.Environment = env
+	fullSrv.Config = daemonConfigRender()
+
+	return SyncResponseETag(true, fullSrv, fullSrv.Config)
 }
 
 func api10Put(d *Daemon, r *http.Request) Response {
@@ -189,7 +222,7 @@ func api10Put(d *Daemon, r *http.Request) Response {
 		return PreconditionFailed(err)
 	}
 
-	req := apiPut{}
+	req := api.ServerPut{}
 	if err := shared.ReadToJSON(r.Body, &req); err != nil {
 		return BadRequest(err)
 	}
@@ -208,7 +241,7 @@ func api10Patch(d *Daemon, r *http.Request) Response {
 		return PreconditionFailed(err)
 	}
 
-	req := apiPut{}
+	req := api.ServerPut{}
 	if err := shared.ReadToJSON(r.Body, &req); err != nil {
 		return BadRequest(err)
 	}
@@ -227,7 +260,7 @@ func api10Patch(d *Daemon, r *http.Request) Response {
 	return doApi10Update(d, oldConfig, req)
 }
 
-func doApi10Update(d *Daemon, oldConfig map[string]string, req apiPut) Response {
+func doApi10Update(d *Daemon, oldConfig map[string]string, req api.ServerPut) Response {
 	// Deal with special keys
 	for k, v := range req.Config {
 		config := daemonConfig[k]
