@@ -10,7 +10,6 @@ import (
 	"strings"
 
 	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/logger"
 
 	log "gopkg.in/inconshreveable/log15.v2"
 )
@@ -80,6 +79,8 @@ const AA_PROFILE_BASE = `
 
   # block some other dangerous paths
   deny @{PROC}/kcore rwklx,
+  deny @{PROC}/kmem rwklx,
+  deny @{PROC}/mem rwklx,
   deny @{PROC}/sysrq-trigger rwklx,
 
   # deny writes in /sys except for /sys/fs/cgroup, also allow
@@ -317,9 +318,14 @@ func getAAProfileContent(c container) string {
 		profile += "  mount fstype=cgroup -> /sys/fs/cgroup/**,\n"
 	}
 
-	if aaStacking && !aaStacked {
+	if aaStacking {
 		profile += "\n  ### Feature: apparmor stacking\n"
-		profile += `  ### Configuration: apparmor profile loading (in namespace)
+
+		if c.IsPrivileged() {
+			profile += "\n  ### Configuration: apparmor loading disabled in privileged containers\n"
+			profile += "  deny /sys/k*{,/**} rwklx,\n"
+		} else {
+			profile += `  ### Configuration: apparmor loading in unprivileged containers
   deny /sys/k[^e]*{,/**} wklx,
   deny /sys/ke[^r]*{,/**} wklx,
   deny /sys/ker[^n]*{,/**} wklx,
@@ -345,8 +351,8 @@ func getAAProfileContent(c container) string {
   deny /sys/kernel/security?*{,/**} wklx,
   deny /sys/kernel?*{,/**} wklx,
 `
-		profile += fmt.Sprintf("  change_profile -> \":%s:*\",\n", AANamespace(c))
-		profile += fmt.Sprintf("  change_profile -> \":%s://*\",\n", AANamespace(c))
+			profile += fmt.Sprintf("  change_profile -> \":%s://*\",\n", AANamespace(c))
+		}
 	} else {
 		profile += "\n  ### Feature: apparmor stacking (not present)\n"
 		profile += "  deny /sys/k*{,/**} rwklx,\n"
@@ -356,12 +362,12 @@ func getAAProfileContent(c container) string {
 		// Apply nesting bits
 		profile += "\n  ### Configuration: nesting\n"
 		profile += strings.TrimLeft(AA_PROFILE_NESTING, "\n")
-		if !aaStacking || aaStacked {
+		if !aaStacking || c.IsPrivileged() {
 			profile += fmt.Sprintf("  change_profile -> \"%s\",\n", AAProfileFull(c))
 		}
 	}
 
-	if !c.IsPrivileged() || runningInUserns {
+	if !c.IsPrivileged() {
 		// Apply unprivileged bits
 		profile += "\n  ### Configuration: unprivileged containers\n"
 		profile += strings.TrimLeft(AA_PROFILE_UNPRIVILEGED, "\n")
@@ -395,7 +401,7 @@ func runApparmor(command string, c container) error {
 	}...)
 
 	if err != nil {
-		logger.Error("Running apparmor",
+		shared.LogError("Running apparmor",
 			log.Ctx{"action": command, "output": output, "err": err})
 	}
 
@@ -403,7 +409,7 @@ func runApparmor(command string, c container) error {
 }
 
 func mkApparmorNamespace(namespace string) error {
-	if !aaStacking || aaStacked {
+	if !aaStacking {
 		return nil
 	}
 
@@ -469,10 +475,10 @@ func AADestroy(c container) error {
 		return nil
 	}
 
-	if aaStacking && !aaStacked {
+	if aaStacking {
 		p := path.Join("/sys/kernel/security/apparmor/policy/namespaces", AANamespace(c))
 		if err := os.Remove(p); err != nil {
-			logger.Error("error removing apparmor namespace", log.Ctx{"err": err, "ns": p})
+			shared.LogError("error removing apparmor namespace", log.Ctx{"err": err, "ns": p})
 		}
 	}
 
@@ -507,7 +513,6 @@ func aaProfile() string {
 	if err == nil {
 		return strings.TrimSpace(string(contents))
 	}
-
 	return ""
 }
 
