@@ -10,14 +10,10 @@ import (
 	"github.com/gorilla/mux"
 
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/version"
 )
 
-type containerSnapshotPostReq struct {
-	Name     string `json:"name"`
-	Stateful bool   `json:"stateful"`
-}
-
 func containerSnapshotsGet(d *Daemon, r *http.Request) Response {
 	recursionStr := r.FormValue("recursion")
 	recursion, err := strconv.Atoi(recursionStr)
@@ -37,7 +33,7 @@ func containerSnapshotsGet(d *Daemon, r *http.Request) Response {
 	}
 
 	resultString := []string{}
-	resultMap := []*shared.SnapshotInfo{}
+	resultMap := []*api.ContainerSnapshot{}
 
 	for _, snap := range snaps {
 		snapName := strings.SplitN(snap.Name(), shared.SnapshotDelimiter, 2)[1]
@@ -50,7 +46,7 @@ func containerSnapshotsGet(d *Daemon, r *http.Request) Response {
 				continue
 			}
 
-			resultMap = append(resultMap, render.(*shared.SnapshotInfo))
+			resultMap = append(resultMap, render.(*api.ContainerSnapshot))
 		}
 	}
 
@@ -68,7 +64,7 @@ func containerSnapshotsGet(d *Daemon, r *http.Request) Response {
 func nextSnapshot(d *Daemon, name string) int {
 	base := name + shared.SnapshotDelimiter + "snap"
 	length := len(base)
-	q := fmt.Sprintf("SELECT MAX(name) FROM containers WHERE type=? AND SUBSTR(name,1,?)=?")
+	q := fmt.Sprintf("SELECT name FROM containers WHERE type=? AND SUBSTR(name,1,?)=?")
 	var numstr string
 	inargs := []interface{}{cTypeSnapshot, length, base}
 	outfmt := []interface{}{numstr}
@@ -111,7 +107,15 @@ func containerSnapshotsPost(d *Daemon, r *http.Request) Response {
 		return SmartError(err)
 	}
 
-	req := containerSnapshotPostReq{}
+	ourStart, err := c.StorageStart()
+	if err != nil {
+		return InternalError(err)
+	}
+	if ourStart {
+		defer c.StorageStop()
+	}
+
+	req := api.ContainerSnapshotsPost{}
 	if err := json.NewDecoder(r.Body).Decode(&req); err != nil {
 		return BadRequest(err)
 	}
@@ -189,7 +193,7 @@ func snapshotGet(sc container, name string) Response {
 		return SmartError(err)
 	}
 
-	return SyncResponse(true, render.(*shared.SnapshotInfo))
+	return SyncResponse(true, render.(*api.ContainerSnapshot))
 }
 
 func snapshotPost(d *Daemon, r *http.Request, sc container, containerName string) Response {
@@ -200,7 +204,7 @@ func snapshotPost(d *Daemon, r *http.Request, sc container, containerName string
 
 	migration, err := raw.GetBool("migration")
 	if err == nil && migration {
-		ws, err := NewMigrationSource(sc)
+		ws, err := NewMigrationSource(sc, false)
 		if err != nil {
 			return SmartError(err)
 		}
