@@ -3,7 +3,9 @@ package main
 import (
 	"fmt"
 
+	"github.com/lxc/lxd/lxd/types"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
 )
 
 func (suite *lxdTestSuite) TestContainer_ProfilesDefault() {
@@ -36,7 +38,7 @@ func (suite *lxdTestSuite) TestContainer_ProfilesMulti() {
 		"unprivileged",
 		"unprivileged",
 		map[string]string{"security.privileged": "true"},
-		shared.Devices{})
+		types.Devices{})
 
 	suite.Req.Nil(err, "Failed to create the unprivileged profile.")
 	defer func() {
@@ -70,8 +72,8 @@ func (suite *lxdTestSuite) TestContainer_ProfilesOverwriteDefaultNic() {
 		Ctype:     cTypeRegular,
 		Ephemeral: false,
 		Config:    map[string]string{"security.privileged": "true"},
-		Devices: shared.Devices{
-			"eth0": shared.Device{
+		Devices: types.Devices{
+			"eth0": types.Device{
 				"type":    "nic",
 				"nictype": "bridged",
 				"parent":  "unknownbr0"}},
@@ -86,7 +88,7 @@ func (suite *lxdTestSuite) TestContainer_ProfilesOverwriteDefaultNic() {
 	out, _, err := c.Render()
 	suite.Req.Nil(err)
 
-	state := out.(*shared.ContainerInfo)
+	state := out.(*api.Container)
 	defer c.Delete()
 
 	suite.Equal(
@@ -100,8 +102,8 @@ func (suite *lxdTestSuite) TestContainer_LoadFromDB() {
 		Ctype:     cTypeRegular,
 		Ephemeral: false,
 		Config:    map[string]string{"security.privileged": "true"},
-		Devices: shared.Devices{
-			"eth0": shared.Device{
+		Devices: types.Devices{
+			"eth0": types.Device{
 				"type":    "nic",
 				"nictype": "bridged",
 				"parent":  "unknownbr0"}},
@@ -117,6 +119,8 @@ func (suite *lxdTestSuite) TestContainer_LoadFromDB() {
 	c2, err := containerLoadByName(suite.d, "testFoo")
 	c2.IsRunning()
 	suite.Req.Nil(err)
+	_, err = c2.StorageStart()
+	suite.Req.Nil(err)
 
 	suite.Exactly(
 		c,
@@ -186,7 +190,6 @@ func (suite *lxdTestSuite) TestContainer_IsPrivileged_Privileged() {
 
 	c, err := containerCreateInternal(suite.d, args)
 	suite.Req.Nil(err)
-	defer c.Delete()
 
 	suite.Req.True(c.IsPrivileged(), "This container should be privileged.")
 	suite.Req.Nil(c.Delete(), "Failed to delete the container.")
@@ -202,7 +205,6 @@ func (suite *lxdTestSuite) TestContainer_IsPrivileged_Unprivileged() {
 
 	c, err := containerCreateInternal(suite.d, args)
 	suite.Req.Nil(err)
-	defer c.Delete()
 
 	suite.Req.False(c.IsPrivileged(), "This container should be unprivileged.")
 	suite.Req.Nil(c.Delete(), "Failed to delete the container.")
@@ -253,14 +255,14 @@ func (suite *lxdTestSuite) TestContainer_findIdmap_isolated() {
 
 	for i := 0; i < 2; i++ {
 		suite.Req.Equal(host.Hostid+65536, map1.Idmap[i].Hostid, "hostids don't match %d", i)
-		suite.Req.Equal(0, map1.Idmap[i].Nsid, "nsid nonzero")
-		suite.Req.Equal(65536, map1.Idmap[i].Maprange, "incorrect maprange")
+		suite.Req.Equal(int64(0), map1.Idmap[i].Nsid, "nsid nonzero")
+		suite.Req.Equal(int64(65536), map1.Idmap[i].Maprange, "incorrect maprange")
 	}
 
 	for i := 0; i < 2; i++ {
 		suite.Req.Equal(host.Hostid+65536*2, map2.Idmap[i].Hostid, "hostids don't match")
-		suite.Req.Equal(0, map2.Idmap[i].Nsid, "nsid nonzero")
-		suite.Req.Equal(65536, map2.Idmap[i].Maprange, "incorrect maprange")
+		suite.Req.Equal(int64(0), map2.Idmap[i].Nsid, "nsid nonzero")
+		suite.Req.Equal(int64(65536), map2.Idmap[i].Maprange, "incorrect maprange")
 	}
 }
 
@@ -294,14 +296,14 @@ func (suite *lxdTestSuite) TestContainer_findIdmap_mixed() {
 
 	for i := 0; i < 2; i++ {
 		suite.Req.Equal(host.Hostid, map1.Idmap[i].Hostid, "hostids don't match %d", i)
-		suite.Req.Equal(0, map1.Idmap[i].Nsid, "nsid nonzero")
+		suite.Req.Equal(int64(0), map1.Idmap[i].Nsid, "nsid nonzero")
 		suite.Req.Equal(host.Maprange, map1.Idmap[i].Maprange, "incorrect maprange")
 	}
 
 	for i := 0; i < 2; i++ {
 		suite.Req.Equal(host.Hostid+65536, map2.Idmap[i].Hostid, "hostids don't match")
-		suite.Req.Equal(0, map2.Idmap[i].Nsid, "nsid nonzero")
-		suite.Req.Equal(65536, map2.Idmap[i].Maprange, "incorrect maprange")
+		suite.Req.Equal(int64(0), map2.Idmap[i].Nsid, "nsid nonzero")
+		suite.Req.Equal(int64(65536), map2.Idmap[i].Maprange, "incorrect maprange")
 	}
 }
 
@@ -324,19 +326,19 @@ func (suite *lxdTestSuite) TestContainer_findIdmap_raw() {
 
 	for _, i := range []int{0, 3} {
 		suite.Req.Equal(host.Hostid, map1.Idmap[i].Hostid, "hostids don't match")
-		suite.Req.Equal(0, map1.Idmap[i].Nsid, "nsid nonzero")
-		suite.Req.Equal(1000, map1.Idmap[i].Maprange, "incorrect maprange")
+		suite.Req.Equal(int64(0), map1.Idmap[i].Nsid, "nsid nonzero")
+		suite.Req.Equal(int64(1000), map1.Idmap[i].Maprange, "incorrect maprange")
 	}
 
 	for _, i := range []int{1, 4} {
-		suite.Req.Equal(1000, map1.Idmap[i].Hostid, "hostids don't match")
-		suite.Req.Equal(1000, map1.Idmap[i].Nsid, "invalid nsid")
-		suite.Req.Equal(1, map1.Idmap[i].Maprange, "incorrect maprange")
+		suite.Req.Equal(int64(1000), map1.Idmap[i].Hostid, "hostids don't match")
+		suite.Req.Equal(int64(1000), map1.Idmap[i].Nsid, "invalid nsid")
+		suite.Req.Equal(int64(1), map1.Idmap[i].Maprange, "incorrect maprange")
 	}
 
 	for _, i := range []int{2, 5} {
 		suite.Req.Equal(host.Hostid+1001, map1.Idmap[i].Hostid, "hostids don't match")
-		suite.Req.Equal(1001, map1.Idmap[i].Nsid, "invalid nsid")
+		suite.Req.Equal(int64(1001), map1.Idmap[i].Nsid, "invalid nsid")
 		suite.Req.Equal(host.Maprange-1000-1, map1.Idmap[i].Maprange, "incorrect maprange")
 	}
 }
