@@ -11,55 +11,15 @@ import (
 	"github.com/dustinkirkland/golang-petname"
 	"github.com/gorilla/websocket"
 
+	"github.com/lxc/lxd/lxd/types"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/osarch"
 
 	log "gopkg.in/inconshreveable/log15.v2"
 )
 
-type containerImageSource struct {
-	Type        string `json:"type"`
-	Certificate string `json:"certificate"`
-
-	/* for "image" type */
-	Alias       string            `json:"alias"`
-	Fingerprint string            `json:"fingerprint"`
-	Properties  map[string]string `json:"properties"`
-	Server      string            `json:"server"`
-	Secret      string            `json:"secret"`
-	Protocol    string            `json:"protocol"`
-
-	/*
-	 * for "migration" and "copy" types, as an optimization users can
-	 * provide an image hash to extract before the filesystem is rsync'd,
-	 * potentially cutting down filesystem transfer time. LXD will not go
-	 * and fetch this image, it will simply use it if it exists in the
-	 * image store.
-	 */
-	BaseImage string `json:"base-image"`
-
-	/* for "migration" type */
-	Mode       string            `json:"mode"`
-	Operation  string            `json:"operation"`
-	Websockets map[string]string `json:"secrets"`
-
-	/* for "copy" type */
-	Source string `json:"source"`
-	/* for "migration" type. Whether the migration is live. */
-	Live bool `json:"live"`
-}
-
-type containerPostReq struct {
-	Architecture string               `json:"architecture"`
-	Config       map[string]string    `json:"config"`
-	Devices      shared.Devices       `json:"devices"`
-	Ephemeral    bool                 `json:"ephemeral"`
-	Name         string               `json:"name"`
-	Profiles     []string             `json:"profiles"`
-	Source       containerImageSource `json:"source"`
-}
-
-func createFromImage(d *Daemon, req *containerPostReq) Response {
+func createFromImage(d *Daemon, req *api.ContainersPost) Response {
 	var hash string
 	var err error
 
@@ -76,8 +36,6 @@ func createFromImage(d *Daemon, req *containerPostReq) Response {
 
 			hash = alias.Target
 		}
-	} else if req.Source.Fingerprint != "" {
-		hash = req.Source.Fingerprint
 	} else if req.Source.Properties != nil {
 		if req.Source.Server != "" {
 			return BadRequest(fmt.Errorf("Property match is only supported for local images"))
@@ -88,15 +46,15 @@ func createFromImage(d *Daemon, req *containerPostReq) Response {
 			return InternalError(err)
 		}
 
-		var image *shared.ImageInfo
+		var image *api.Image
 
-		for _, hash := range hashes {
-			_, img, err := dbImageGet(d.db, hash, false, true)
+		for _, imageHash := range hashes {
+			_, img, err := dbImageGet(d.db, imageHash, false, true)
 			if err != nil {
 				continue
 			}
 
-			if image != nil && img.CreationDate.Before(image.CreationDate) {
+			if image != nil && img.CreatedAt.Before(image.CreatedAt) {
 				continue
 			}
 
@@ -125,10 +83,19 @@ func createFromImage(d *Daemon, req *containerPostReq) Response {
 	}
 
 	run := func(op *operation) error {
+		args := containerArgs{
+			Config:    req.Config,
+			Ctype:     cTypeRegular,
+			Devices:   req.Devices,
+			Ephemeral: req.Ephemeral,
+			Name:      req.Name,
+			Profiles:  req.Profiles,
+		}
+
 		if req.Source.Server != "" {
 			hash, err = d.ImageDownload(
 				op, req.Source.Server, req.Source.Protocol, req.Source.Certificate, req.Source.Secret,
-				hash, true, daemonConfig["images.auto_update_cached"].GetBool())
+				hash, true, daemonConfig["images.auto_update_cached"].GetBool(), "")
 			if err != nil {
 				return err
 			}
@@ -139,25 +106,12 @@ func createFromImage(d *Daemon, req *containerPostReq) Response {
 			return err
 		}
 
-		hash = imgInfo.Fingerprint
-
-		architecture, err := osarch.ArchitectureId(imgInfo.Architecture)
+		args.Architecture, err = osarch.ArchitectureId(imgInfo.Architecture)
 		if err != nil {
-			architecture = 0
+			return err
 		}
 
-		args := containerArgs{
-			Architecture: architecture,
-			BaseImage:    hash,
-			Config:       req.Config,
-			Ctype:        cTypeRegular,
-			Devices:      req.Devices,
-			Ephemeral:    req.Ephemeral,
-			Name:         req.Name,
-			Profiles:     req.Profiles,
-		}
-
-		_, err = containerCreateFromImage(d, args, hash)
+		_, err = containerCreateFromImage(d, args, imgInfo.Fingerprint)
 		return err
 	}
 
@@ -172,20 +126,22 @@ func createFromImage(d *Daemon, req *containerPostReq) Response {
 	return OperationResponse(op)
 }
 
-func createFromNone(d *Daemon, req *containerPostReq) Response {
-	architecture, err := osarch.ArchitectureId(req.Architecture)
-	if err != nil {
-		architecture = 0
+func createFromNone(d *Daemon, req *api.ContainersPost) Response {
+	args := containerArgs{
+		Config:    req.Config,
+		Ctype:     cTypeRegular,
+		Devices:   req.Devices,
+		Ephemeral: req.Ephemeral,
+		Name:      req.Name,
+		Profiles:  req.Profiles,
 	}
 
-	args := containerArgs{
-		Architecture: architecture,
-		Config:       req.Config,
-		Ctype:        cTypeRegular,
-		Devices:      req.Devices,
-		Ephemeral:    req.Ephemeral,
-		Name:         req.Name,
-		Profiles:     req.Profiles,
+	if req.Architecture != "" {
+		architecture, err := osarch.ArchitectureId(req.Architecture)
+		if err != nil {
+			return InternalError(err)
+		}
+		args.Architecture = architecture
 	}
 
 	run := func(op *operation) error {
@@ -204,16 +160,21 @@ func createFromNone(d *Daemon, req *containerPostReq) Response {
 	return OperationResponse(op)
 }
 
-func createFromMigration(d *Daemon, req *containerPostReq) Response {
+func createFromMigration(d *Daemon, req *api.ContainersPost) Response {
+	// Validate migration mode
 	if req.Source.Mode != "pull" && req.Source.Mode != "push" {
 		return NotImplemented
 	}
 
+	var c container
+
+	// Parse the architecture name
 	architecture, err := osarch.ArchitectureId(req.Architecture)
 	if err != nil {
-		architecture = 0
+		return BadRequest(err)
 	}
 
+	// Prepare the container creation request
 	args := containerArgs{
 		Architecture: architecture,
 		BaseImage:    req.Source.BaseImage,
@@ -225,8 +186,85 @@ func createFromMigration(d *Daemon, req *containerPostReq) Response {
 		Profiles:     req.Profiles,
 	}
 
-	var c container
-	_, _, err = dbImageGet(d.db, req.Source.BaseImage, false, true)
+	// Grab the container's root device if one is specified
+	storagePool := ""
+	storagePoolProfile := ""
+
+	localRootDiskDeviceKey, localRootDiskDevice, _ := containerGetRootDiskDevice(req.Devices)
+	if localRootDiskDeviceKey != "" {
+		storagePool = localRootDiskDevice["pool"]
+	}
+
+	// Handle copying/moving between two storage-api LXD instances.
+	if storagePool != "" {
+		_, err := dbStoragePoolGetID(d.db, storagePool)
+		if err == NoSuchObjectError {
+			storagePool = ""
+			// Unset the local root disk device storage pool if not
+			// found.
+			localRootDiskDevice["pool"] = ""
+		}
+	}
+
+	// If we don't have a valid pool yet, look through profiles
+	if storagePool == "" {
+		for _, pName := range req.Profiles {
+			_, p, err := dbProfileGet(d.db, pName)
+			if err != nil {
+				return InternalError(err)
+			}
+
+			k, v, _ := containerGetRootDiskDevice(p.Devices)
+			if k != "" && v["pool"] != "" {
+				// Keep going as we want the last one in the profile chain
+				storagePool = v["pool"]
+				storagePoolProfile = pName
+			}
+		}
+	}
+
+	// If there is just a single pool in the database, use that
+	if storagePool == "" {
+		pools, err := dbStoragePools(d.db)
+		if err != nil {
+			return InternalError(err)
+		}
+
+		if len(pools) == 1 {
+			storagePool = pools[0]
+		}
+	}
+
+	if storagePool == "" {
+		return BadRequest(fmt.Errorf("Can't find a storage pool for the container to use"))
+	}
+
+	if localRootDiskDeviceKey == "" && storagePoolProfile == "" {
+		// Give the container it's own local root disk device with a
+		// pool property.
+		rootDev := map[string]string{}
+		rootDev["type"] = "disk"
+		rootDev["path"] = "/"
+		rootDev["pool"] = storagePool
+		if args.Devices == nil {
+			args.Devices = map[string]map[string]string{}
+		}
+
+		// Make sure that we do not overwrite a device the user
+		// is currently using under the name "root".
+		rootDevName := "root"
+		for i := 0; i < 100; i++ {
+			if args.Devices[rootDevName] == nil {
+				break
+			}
+			rootDevName = fmt.Sprintf("root%d", i)
+			continue
+		}
+
+		args.Devices["root"] = rootDev
+	} else if localRootDiskDeviceKey != "" && localRootDiskDevice["pool"] == "" {
+		args.Devices[localRootDiskDeviceKey]["pool"] = storagePool
+	}
 
 	/* Only create a container from an image if we're going to
 	 * rsync over the top of it. In the case of a better file
@@ -241,16 +279,46 @@ func createFromMigration(d *Daemon, req *containerPostReq) Response {
 	 * point and just negotiate it over the migration control
 	 * socket. Anyway, it'll happen later :)
 	 */
-	if err == nil && d.Storage.MigrationType() == MigrationFSType_RSYNC {
-		c, err = containerCreateFromImage(d, args, req.Source.BaseImage)
+	_, _, err = dbImageGet(d.db, req.Source.BaseImage, false, true)
+	if err != nil {
+		c, err = containerCreateAsEmpty(d, args)
 		if err != nil {
 			return InternalError(err)
 		}
 	} else {
-		c, err = containerCreateAsEmpty(d, args)
+		// Retrieve the future storage pool
+		cM, err := containerLXCLoad(d, args)
 		if err != nil {
 			return InternalError(err)
 		}
+
+		_, rootDiskDevice, err := containerGetRootDiskDevice(cM.ExpandedDevices())
+		if err != nil {
+			return InternalError(err)
+		}
+
+		if rootDiskDevice["pool"] == "" {
+			return BadRequest(fmt.Errorf("The container's root device is missing the pool property."))
+		}
+
+		storagePool = rootDiskDevice["pool"]
+
+		ps, err := storagePoolInit(d, storagePool)
+		if err != nil {
+			return InternalError(err)
+		}
+
+		if ps.MigrationType() == MigrationFSType_RSYNC {
+			c, err = containerCreateFromImage(d, args, req.Source.BaseImage)
+			if err != nil {
+				return InternalError(err)
+			}
+		} else {
+			c, err = containerCreateAsEmpty(d, args)
+			if err != nil {
+				return InternalError(err)
+			}
+		}
 	}
 
 	var cert *x509.Certificate
@@ -297,7 +365,7 @@ func createFromMigration(d *Daemon, req *containerPostReq) Response {
 	}
 
 	run := func(op *operation) error {
-		// And finaly run the migration.
+		// And finally run the migration.
 		err = sink.Do(op)
 		if err != nil {
 			shared.LogError("Error during migration sink", log.Ctx{"err": err})
@@ -333,7 +401,7 @@ func createFromMigration(d *Daemon, req *containerPostReq) Response {
 	return OperationResponse(op)
 }
 
-func createFromCopy(d *Daemon, req *containerPostReq) Response {
+func createFromCopy(d *Daemon, req *api.ContainersPost) Response {
 	if req.Source.Source == "" {
 		return BadRequest(fmt.Errorf("must specify a source container"))
 	}
@@ -365,6 +433,22 @@ func createFromCopy(d *Daemon, req *containerPostReq) Response {
 		req.Config[key] = value
 	}
 
+	// Devices override
+	sourceDevices := source.LocalDevices()
+
+	if req.Devices == nil {
+		req.Devices = make(map[string]map[string]string)
+	}
+
+	for key, value := range sourceDevices {
+		_, exists := req.Devices[key]
+		if exists {
+			continue
+		}
+
+		req.Devices[key] = value
+	}
+
 	// Profiles override
 	if req.Profiles == nil {
 		req.Profiles = source.Profiles()
@@ -375,7 +459,7 @@ func createFromCopy(d *Daemon, req *containerPostReq) Response {
 		BaseImage:    req.Source.BaseImage,
 		Config:       req.Config,
 		Ctype:        cTypeRegular,
-		Devices:      source.LocalDevices(),
+		Devices:      req.Devices,
 		Ephemeral:    req.Ephemeral,
 		Name:         req.Name,
 		Profiles:     req.Profiles,
@@ -404,11 +488,17 @@ func createFromCopy(d *Daemon, req *containerPostReq) Response {
 func containersPost(d *Daemon, r *http.Request) Response {
 	shared.LogDebugf("Responding to container create")
 
-	req := containerPostReq{}
+	req := api.ContainersPost{}
 	if err := json.NewDecoder(r.Body).Decode(&req); err != nil {
 		return BadRequest(err)
 	}
 
+	// If no storage pool is found, error out.
+	pools, err := dbStoragePools(d.db)
+	if err != nil || len(pools) == 0 {
+		return BadRequest(fmt.Errorf("No storage pool found. Please create a new storage pool."))
+	}
+
 	if req.Name == "" {
 		cs, err := dbContainersList(d.db, cTypeRegular)
 		if err != nil {
@@ -431,7 +521,7 @@ func containersPost(d *Daemon, r *http.Request) Response {
 	}
 
 	if req.Devices == nil {
-		req.Devices = shared.Devices{}
+		req.Devices = types.Devices{}
 	}
 
 	if req.Config == nil {
