@@ -31,6 +31,7 @@ import (
 
 	"github.com/lxc/lxd"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/logging"
 	"github.com/lxc/lxd/shared/osarch"
 	"github.com/lxc/lxd/shared/version"
@@ -79,8 +80,6 @@ type Daemon struct {
 	shutdownChan        chan bool
 	resetAutoUpdateChan chan bool
 
-	Storage storage
-
 	TCPSocket  *Socket
 	UnixSocket *Socket
 
@@ -89,9 +88,6 @@ type Daemon struct {
 	MockMode  bool
 	SetupMode bool
 
-	imagesDownloading     map[string]chan bool
-	imagesDownloadingLock sync.RWMutex
-
 	tlsConfig *tls.Config
 
 	proxy func(req *http.Request) (*url.URL, error)
@@ -141,10 +137,18 @@ func (d *Daemon) httpClient(certificate string) (*http.Client, error) {
 		Transport: tr,
 	}
 
+	// Setup redirect policy
+	myhttp.CheckRedirect = func(req *http.Request, via []*http.Request) error {
+		// Replicate the headers
+		req.Header = via[len(via)-1].Header
+
+		return nil
+	}
+
 	return &myhttp, nil
 }
 
-func (d *Daemon) httpGetSync(url string, certificate string) (*lxd.Response, error) {
+func (d *Daemon) httpGetSync(url string, certificate string) (*api.Response, error) {
 	var err error
 
 	req, err := http.NewRequest("GET", url, nil)
@@ -169,7 +173,7 @@ func (d *Daemon) httpGetSync(url string, certificate string) (*lxd.Response, err
 		return nil, err
 	}
 
-	if resp.Type != lxd.Sync {
+	if resp.Type != api.SyncResponse {
 		return nil, fmt.Errorf("unexpected non-sync response")
 	}
 
@@ -197,7 +201,7 @@ func (d *Daemon) httpGetFile(url string, certificate string) (*http.Response, er
 	}
 
 	if raw.StatusCode != 200 {
-		_, err := lxd.HoistResponse(raw, lxd.Error)
+		_, err := lxd.HoistResponse(raw, api.ErrorResponse)
 		if err != nil {
 			return nil, err
 		}
@@ -351,38 +355,73 @@ func (d *Daemon) createCmd(version string, c Command) {
 	})
 }
 
-func (d *Daemon) SetupStorageDriver() error {
-	var err error
-
-	lvmVgName := daemonConfig["storage.lvm_vg_name"].Get()
-	zfsPoolName := daemonConfig["storage.zfs_pool_name"].Get()
-
-	if lvmVgName != "" {
-		d.Storage, err = newStorage(d, storageTypeLvm)
-		if err != nil {
-			shared.LogErrorf("Could not initialize storage type LVM: %s - falling back to dir", err)
-		} else {
+func (d *Daemon) SetupStorageDriver(forceCheck bool) error {
+	pools, err := dbStoragePools(d.db)
+	if err != nil {
+		if err == NoSuchObjectError {
+			shared.LogDebugf("No existing storage pools detected.")
 			return nil
 		}
-	} else if zfsPoolName != "" {
-		d.Storage, err = newStorage(d, storageTypeZfs)
+		shared.LogDebugf("Failed to retrieve existing storage pools.")
+		return err
+	}
+
+	// In case the daemon got killed during upgrade we will already have a
+	// valid storage pool entry but it might have gotten messed up and so we
+	// cannot perform StoragePoolCheck(). This case can be detected by
+	// looking at the patches db: If we already have a storage pool defined
+	// but the upgrade somehow got messed up then there will be no
+	// "storage_api" entry in the db.
+	if len(pools) > 0 && !forceCheck {
+		appliedPatches, err := dbPatches(d.db)
 		if err != nil {
-			shared.LogErrorf("Could not initialize storage type ZFS: %s - falling back to dir", err)
-		} else {
+			return err
+		}
+
+		if !shared.StringInSlice("storage_api", appliedPatches) {
+			shared.LogWarnf("Incorrectly applied \"storage_api\" patch. Skipping storage pool initialization as it might be corrupt.")
 			return nil
 		}
-	} else if d.BackingFs == "btrfs" {
-		d.Storage, err = newStorage(d, storageTypeBtrfs)
+
+	}
+
+	for _, pool := range pools {
+		shared.LogDebugf("Initializing and checking storage pool \"%s\".", pool)
+		s, err := storagePoolInit(d, pool)
 		if err != nil {
-			shared.LogErrorf("Could not initialize storage type btrfs: %s - falling back to dir", err)
-		} else {
-			return nil
+			shared.LogErrorf("Error initializing storage pool \"%s\": %s. Correct functionality of the storage pool cannot be guaranteed.", pool, err)
+			continue
+		}
+
+		err = s.StoragePoolCheck()
+		if err != nil {
+			return err
 		}
 	}
 
-	d.Storage, err = newStorage(d, storageTypeDir)
+	// Get a list of all storage drivers currently in use
+	// on this LXD instance. Only do this when we do not already have done
+	// this once to avoid unnecessarily querying the db. All subsequent
+	// updates of the cache will be done when we create or delete storage
+	// pools in the db. Since this is a rare event, this cache
+	// implementation is a classic frequent-read, rare-update case so
+	// copy-on-write semantics without locking in the read case seems
+	// appropriate. (Should be cheaper then querying the db all the time,
+	// especially if we keep adding more storage drivers.)
+	if !storagePoolDriversCacheInitialized {
+		tmp, err := dbStoragePoolsGetDrivers(d.db)
+		if err != nil && err != NoSuchObjectError {
+			return nil
+		}
 
-	return err
+		storagePoolDriversCacheLock.Lock()
+		storagePoolDriversCacheVal.Store(tmp)
+		storagePoolDriversCacheLock.Unlock()
+
+		storagePoolDriversCacheInitialized = true
+	}
+
+	return nil
 }
 
 // have we setup shared mounts?
@@ -390,35 +429,28 @@ var sharedMounted bool
 var sharedMountsLock sync.Mutex
 
 func setupSharedMounts() error {
+	// Check if we already went through this
 	if sharedMounted {
 		return nil
 	}
 
+	// Get a lock to prevent races
 	sharedMountsLock.Lock()
 	defer sharedMountsLock.Unlock()
 
-	if sharedMounted {
-		return nil
-	}
-
+	// Check if already setup
 	path := shared.VarPath("shmounts")
-
-	isShared, err := shared.IsOnSharedMount(path)
-	if err != nil {
-		return err
-	}
-
-	if isShared {
-		// / may already be ms-shared, or shmounts may have
-		// been mounted by a previous lxd run
+	if shared.IsMountPoint(path) {
 		sharedMounted = true
 		return nil
 	}
 
-	if err := syscall.Mount(path, path, "none", syscall.MS_BIND, ""); err != nil {
+	// Mount a new tmpfs
+	if err := syscall.Mount("tmpfs", path, "tmpfs", 0, "size=100k,mode=0711"); err != nil {
 		return err
 	}
 
+	// Mark as MS_SHARED and MS_REC
 	var flags uintptr = syscall.MS_SHARED | syscall.MS_REC
 	if err := syscall.Mount(path, path, "none", flags, ""); err != nil {
 		return err
@@ -544,8 +576,6 @@ func haveMacAdmin() bool {
 
 func (d *Daemon) Init() error {
 	/* Initialize some variables */
-	d.imagesDownloading = map[string]chan bool{}
-
 	d.readyChan = make(chan bool)
 	d.shutdownChan = make(chan bool)
 
@@ -768,6 +798,15 @@ func (d *Daemon) Init() error {
 	if err := os.MkdirAll(shared.VarPath("snapshots"), 0700); err != nil {
 		return err
 	}
+	if err := os.MkdirAll(shared.VarPath("networks"), 0711); err != nil {
+		return err
+	}
+	if err := os.MkdirAll(shared.VarPath("disks"), 0700); err != nil {
+		return err
+	}
+	if err := os.MkdirAll(shared.VarPath("storage-pools"), 0711); err != nil {
+		return err
+	}
 
 	/* Detect the filesystem */
 	d.BackingFs, err = filesystemDetect(d.lxcpath)
@@ -778,12 +817,42 @@ func (d *Daemon) Init() error {
 	/* Read the uid/gid allocation */
 	d.IdmapSet, err = shared.DefaultIdmapSet()
 	if err != nil {
-		shared.LogWarn("Error reading idmap", log.Ctx{"err": err.Error()})
+		shared.LogWarn("Error reading default uid/gid map", log.Ctx{"err": err.Error()})
 		shared.LogWarnf("Only privileged containers will be able to run")
+		d.IdmapSet = nil
 	} else {
-		shared.LogInfof("Default uid/gid map:")
-		for _, lxcmap := range d.IdmapSet.ToLxcString() {
-			shared.LogInfof(strings.TrimRight(" - "+lxcmap, "\n"))
+		kernelIdmapSet, err := shared.CurrentIdmapSet()
+		if err == nil {
+			shared.LogInfof("Kernel uid/gid map:")
+			for _, lxcmap := range kernelIdmapSet.ToLxcString() {
+				shared.LogInfof(strings.TrimRight(" - "+lxcmap, "\n"))
+			}
+		}
+
+		if len(d.IdmapSet.Idmap) == 0 {
+			shared.LogWarnf("No available uid/gid map could be found")
+			shared.LogWarnf("Only privileged containers will be able to run")
+			d.IdmapSet = nil
+		} else {
+			shared.LogInfof("Configured LXD uid/gid map:")
+			for _, lxcmap := range d.IdmapSet.Idmap {
+				suffix := ""
+
+				if lxcmap.Usable() != nil {
+					suffix = " (unusable)"
+				}
+
+				for _, lxcEntry := range lxcmap.ToLxcString() {
+					shared.LogInfof(" - %s%s", strings.TrimRight(lxcEntry, "\n"), suffix)
+				}
+			}
+
+			err = d.IdmapSet.Usable()
+			if err != nil {
+				shared.LogWarnf("One or more uid/gid map entry isn't usable (typically due to nesting)")
+				shared.LogWarnf("Only privileged containers will be able to run")
+				d.IdmapSet = nil
+			}
 		}
 	}
 
@@ -800,10 +869,10 @@ func (d *Daemon) Init() error {
 	}
 
 	if !d.MockMode {
-		/* Setup the storage driver */
-		err = d.SetupStorageDriver()
+		/* Read the storage pools */
+		err = d.SetupStorageDriver(false)
 		if err != nil {
-			return fmt.Errorf("Failed to setup storage: %s", err)
+			return err
 		}
 
 		/* Apply all patches */
@@ -848,6 +917,17 @@ func (d *Daemon) Init() error {
 		daemonConfig["core.proxy_ignore_hosts"].Get(),
 	)
 
+	/* Setup some mounts (nice to have) */
+	if !d.MockMode {
+		// Attempt to mount the shmounts tmpfs
+		setupSharedMounts()
+
+		// Attempt to Mount the devlxd tmpfs
+		if !shared.IsMountPoint(shared.VarPath("devlxd")) {
+			syscall.Mount("tmpfs", shared.VarPath("devlxd"), "tmpfs", 0, "size=100k,mode=0755")
+		}
+	}
+
 	/* Setup /dev/lxd */
 	shared.LogInfof("Starting /dev/lxd handler")
 	d.devlxd, err = createAndBindDevLxd()
@@ -855,6 +935,11 @@ func (d *Daemon) Init() error {
 		return err
 	}
 
+	d.tomb.Go(func() error {
+		server := devLxdServer(d)
+		return server.Serve(d.devlxd)
+	})
+
 	if !d.MockMode {
 		/* Start the scheduler */
 		go deviceEventListener(d)
@@ -925,6 +1010,7 @@ func (d *Daemon) Init() error {
 		NotFound.Render(w)
 	})
 
+	// Prepare the list of listeners
 	listeners := d.GetListeners()
 	if len(listeners) > 0 {
 		shared.LogInfof("LXD is socket activated")
@@ -1009,25 +1095,19 @@ func (d *Daemon) Init() error {
 		}
 	}
 
-	d.tomb.Go(func() error {
-		shared.LogInfof("REST API daemon:")
-		if d.UnixSocket != nil {
-			shared.LogInfo(" - binding Unix socket", log.Ctx{"socket": d.UnixSocket.Socket.Addr()})
-			d.tomb.Go(func() error { return http.Serve(d.UnixSocket.Socket, &lxdHttpServer{d.mux, d}) })
-		}
+	// Bind the REST API
+	shared.LogInfof("REST API daemon:")
+	if d.UnixSocket != nil {
+		shared.LogInfo(" - binding Unix socket", log.Ctx{"socket": d.UnixSocket.Socket.Addr()})
+		d.tomb.Go(func() error { return http.Serve(d.UnixSocket.Socket, &lxdHttpServer{d.mux, d}) })
+	}
 
-		if d.TCPSocket != nil {
-			shared.LogInfo(" - binding TCP socket", log.Ctx{"socket": d.TCPSocket.Socket.Addr()})
-			d.tomb.Go(func() error { return http.Serve(d.TCPSocket.Socket, &lxdHttpServer{d.mux, d}) })
-		}
-
-		d.tomb.Go(func() error {
-			server := devLxdServer(d)
-			return server.Serve(d.devlxd)
-		})
-		return nil
-	})
+	if d.TCPSocket != nil {
+		shared.LogInfo(" - binding TCP socket", log.Ctx{"socket": d.TCPSocket.Socket.Addr()})
+		d.tomb.Go(func() error { return http.Serve(d.TCPSocket.Socket, &lxdHttpServer{d.mux, d}) })
+	}
 
+	// Run the post initialization actions
 	if !d.MockMode && !d.SetupMode {
 		err := d.Ready()
 		if err != nil {
@@ -1155,23 +1235,24 @@ func (d *Daemon) Stop() error {
 		}
 	}
 
-	if n, err := d.numRunningContainers(); err != nil || n == 0 {
-		shared.LogInfof("Unmounting shmounts")
+	shared.LogInfof("Stopping /dev/lxd handler")
+	d.devlxd.Close()
+	shared.LogInfof("Stopped /dev/lxd handler")
 
+	if n, err := d.numRunningContainers(); err != nil || n == 0 {
+		shared.LogInfof("Unmounting temporary filesystems")
+
+		syscall.Unmount(shared.VarPath("devlxd"), syscall.MNT_DETACH)
 		syscall.Unmount(shared.VarPath("shmounts"), syscall.MNT_DETACH)
 
-		shared.LogInfof("Done unmounting shmounts")
+		shared.LogInfof("Done unmounting temporary filesystems")
 	} else {
-		shared.LogDebugf("Not unmounting shmounts (containers are still running)")
+		shared.LogDebugf("Not unmounting temporary filesystems (containers are still running)")
 	}
 
 	shared.LogInfof("Closing the database")
 	d.db.Close()
 
-	shared.LogInfof("Stopping /dev/lxd handler")
-	d.devlxd.Close()
-	shared.LogInfof("Stopped /dev/lxd handler")
-
 	shared.LogInfof("Saving simplestreams cache")
 	imageSaveStreamCache()
 	shared.LogInfof("Saved simplestreams cache")
