@@ -173,29 +173,30 @@ func (k *daemonConfigKey) GetInt64() int64 {
 func daemonConfigInit(db *sql.DB) error {
 	// Set all the keys
 	daemonConfig = map[string]*daemonConfigKey{
-		"core.https_address":             &daemonConfigKey{valueType: "string", setter: daemonConfigSetAddress},
-		"core.https_allowed_headers":     &daemonConfigKey{valueType: "string"},
-		"core.https_allowed_methods":     &daemonConfigKey{valueType: "string"},
-		"core.https_allowed_origin":      &daemonConfigKey{valueType: "string"},
-		"core.https_allowed_credentials": &daemonConfigKey{valueType: "bool"},
-		"core.proxy_http":                &daemonConfigKey{valueType: "string", setter: daemonConfigSetProxy},
-		"core.proxy_https":               &daemonConfigKey{valueType: "string", setter: daemonConfigSetProxy},
-		"core.proxy_ignore_hosts":        &daemonConfigKey{valueType: "string", setter: daemonConfigSetProxy},
-		"core.trust_password":            &daemonConfigKey{valueType: "string", hiddenValue: true, setter: daemonConfigSetPassword},
+		"core.https_address":             {valueType: "string", setter: daemonConfigSetAddress},
+		"core.https_allowed_headers":     {valueType: "string"},
+		"core.https_allowed_methods":     {valueType: "string"},
+		"core.https_allowed_origin":      {valueType: "string"},
+		"core.https_allowed_credentials": {valueType: "bool"},
+		"core.proxy_http":                {valueType: "string", setter: daemonConfigSetProxy},
+		"core.proxy_https":               {valueType: "string", setter: daemonConfigSetProxy},
+		"core.proxy_ignore_hosts":        {valueType: "string", setter: daemonConfigSetProxy},
+		"core.trust_password":            {valueType: "string", hiddenValue: true, setter: daemonConfigSetPassword},
 
-		"images.auto_update_cached":    &daemonConfigKey{valueType: "bool", defaultValue: "true"},
-		"images.auto_update_interval":  &daemonConfigKey{valueType: "int", defaultValue: "6"},
-		"images.compression_algorithm": &daemonConfigKey{valueType: "string", validator: daemonConfigValidateCompression, defaultValue: "gzip"},
-		"images.remote_cache_expiry":   &daemonConfigKey{valueType: "int", defaultValue: "10", trigger: daemonConfigTriggerExpiry},
+		"images.auto_update_cached":    {valueType: "bool", defaultValue: "true"},
+		"images.auto_update_interval":  {valueType: "int", defaultValue: "6"},
+		"images.compression_algorithm": {valueType: "string", validator: daemonConfigValidateCompression, defaultValue: "gzip"},
+		"images.remote_cache_expiry":   {valueType: "int", defaultValue: "10", trigger: daemonConfigTriggerExpiry},
 
-		"storage.lvm_fstype":           &daemonConfigKey{valueType: "string", defaultValue: "ext4", validValues: []string{"ext4", "xfs"}},
-		"storage.lvm_mount_options":    &daemonConfigKey{valueType: "string", defaultValue: "discard"},
-		"storage.lvm_thinpool_name":    &daemonConfigKey{valueType: "string", defaultValue: "LXDPool", validator: storageLVMValidateThinPoolName},
-		"storage.lvm_vg_name":          &daemonConfigKey{valueType: "string", validator: storageLVMValidateVolumeGroupName, setter: daemonConfigSetStorage},
-		"storage.lvm_volume_size":      &daemonConfigKey{valueType: "string", defaultValue: "10GiB"},
-		"storage.zfs_pool_name":        &daemonConfigKey{valueType: "string", validator: storageZFSValidatePoolName, setter: daemonConfigSetStorage},
-		"storage.zfs_remove_snapshots": &daemonConfigKey{valueType: "bool"},
-		"storage.zfs_use_refquota":     &daemonConfigKey{valueType: "bool"},
+		// Keys deprecated since the implementation of the storage api.
+		"storage.lvm_fstype":           {valueType: "string", defaultValue: "ext4", validValues: []string{"ext4", "xfs"}, validator: storageDeprecatedKeys},
+		"storage.lvm_mount_options":    {valueType: "string", defaultValue: "discard", validator: storageDeprecatedKeys},
+		"storage.lvm_thinpool_name":    {valueType: "string", defaultValue: "LXDPool", validator: storageDeprecatedKeys},
+		"storage.lvm_vg_name":          {valueType: "string", validator: storageDeprecatedKeys},
+		"storage.lvm_volume_size":      {valueType: "string", defaultValue: "10GiB", validator: storageDeprecatedKeys},
+		"storage.zfs_pool_name":        {valueType: "string", validator: storageDeprecatedKeys},
+		"storage.zfs_remove_snapshots": {valueType: "bool", validator: storageDeprecatedKeys},
+		"storage.zfs_use_refquota":     {valueType: "bool", validator: storageDeprecatedKeys},
 	}
 
 	// Load the values from the DB
@@ -260,28 +261,6 @@ func daemonConfigSetPassword(d *Daemon, key string, value string) (string, error
 	return value, nil
 }
 
-func daemonConfigSetStorage(d *Daemon, key string, value string) (string, error) {
-	// The storage driver looks at daemonConfig so just set it temporarily
-	daemonConfigLock.Lock()
-	oldValue := daemonConfig[key].Get()
-	daemonConfig[key].currentValue = value
-	daemonConfigLock.Unlock()
-
-	defer func() {
-		daemonConfigLock.Lock()
-		daemonConfig[key].currentValue = oldValue
-		daemonConfigLock.Unlock()
-	}()
-
-	// Update the current storage driver
-	err := d.SetupStorageDriver()
-	if err != nil {
-		return "", err
-	}
-
-	return value, nil
-}
-
 func daemonConfigSetAddress(d *Daemon, key string, value string) (string, error) {
 	// Update the current https address
 	err := d.UpdateHTTPsPort(value)
@@ -311,7 +290,7 @@ func daemonConfigSetProxy(d *Daemon, key string, value string) (string, error) {
 
 	// Clear the simplestreams cache as it's tied to the old proxy config
 	imageStreamCacheLock.Lock()
-	for k, _ := range imageStreamCache {
+	for k := range imageStreamCache {
 		delete(imageStreamCache, k)
 	}
 	imageStreamCacheLock.Unlock()
@@ -332,3 +311,11 @@ func daemonConfigValidateCompression(d *Daemon, key string, value string) error
 	_, err := exec.LookPath(value)
 	return err
 }
+
+func storageDeprecatedKeys(d *Daemon, key string, value string) error {
+	if value == "" || daemonConfig[key].defaultValue == value {
+		return nil
+	}
+
+	return fmt.Errorf("Setting the key \"%s\" is deprecated in favor of storage pool configuration.", key)
+}
