@@ -13,7 +13,6 @@ import (
 	log "gopkg.in/inconshreveable/log15.v2"
 
 	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/logger"
 )
 
 type eventsHandler struct {
@@ -51,8 +50,7 @@ type eventListener struct {
 	messageTypes []string
 	active       chan bool
 	id           string
-	lock         sync.Mutex
-	done         bool
+	msgLock      sync.Mutex
 }
 
 type eventsServe struct {
@@ -89,10 +87,17 @@ func eventsSocket(r *http.Request, w http.ResponseWriter) error {
 	eventListeners[listener.id] = &listener
 	eventsLock.Unlock()
 
-	logger.Debugf("New events listener: %s", listener.id)
+	shared.LogDebugf("New events listener: %s", listener.id)
 
 	<-listener.active
 
+	eventsLock.Lock()
+	delete(eventListeners, listener.id)
+	eventsLock.Unlock()
+
+	listener.connection.Close()
+	shared.LogDebugf("Disconnected events listener: %s", listener.id)
+
 	return nil
 }
 
@@ -121,32 +126,16 @@ func eventSend(eventType string, eventMessage interface{}) error {
 		}
 
 		go func(listener *eventListener, body []byte) {
-			// Check that the listener still exists
 			if listener == nil {
 				return
 			}
 
-			// Ensure there is only a single even going out at the time
-			listener.lock.Lock()
-			defer listener.lock.Unlock()
-
-			// Make sure we're not done already
-			if listener.done {
-				return
-			}
-
+			listener.msgLock.Lock()
 			err = listener.connection.WriteMessage(websocket.TextMessage, body)
-			if err != nil {
-				// Remove the listener from the list
-				eventsLock.Lock()
-				delete(eventListeners, listener.id)
-				eventsLock.Unlock()
+			listener.msgLock.Unlock()
 
-				// Disconnect the listener
-				listener.connection.Close()
+			if err != nil {
 				listener.active <- false
-				listener.done = true
-				logger.Debugf("Disconnected events listener: %s", listener.id)
 			}
 		}(listener, body)
 	}
