@@ -1,18 +1,22 @@
 package main
 
 import (
-	"sync"
+	"fmt"
+	"os"
 
 	"github.com/lxc/lxd"
 	"github.com/lxc/lxd/shared"
 )
 
 func cmdActivateIfNeeded() error {
+	// Only root should run this
+	if os.Geteuid() != 0 {
+		return fmt.Errorf("This must be run as root")
+	}
+
 	// Don't start a full daemon, we just need DB access
 	d := &Daemon{
-		imagesDownloading:     map[string]chan bool{},
-		imagesDownloadingLock: sync.RWMutex{},
-		lxcpath:               shared.VarPath("containers"),
+		lxcpath: shared.VarPath("containers"),
 	}
 
 	if !shared.PathExists(shared.VarPath("lxd.db")) {
@@ -39,12 +43,13 @@ func cmdActivateIfNeeded() error {
 		return err
 	}
 
-	// Look for auto-started or previously started containers
+	// Load the idmap for unprivileged containers
 	d.IdmapSet, err = shared.DefaultIdmapSet()
 	if err != nil {
 		return err
 	}
 
+	// Look for auto-started or previously started containers
 	result, err := dbContainersList(d.db, cTypeRegular)
 	if err != nil {
 		return err
