@@ -11,6 +11,7 @@ import (
 	"net/http"
 	"net/url"
 	"os"
+	"os/exec"
 	"path/filepath"
 	"strings"
 	"sync"
@@ -151,7 +152,7 @@ type migrationSourceWs struct {
 	allConnected chan bool
 }
 
-func NewMigrationSource(c container) (*migrationSourceWs, error) {
+func NewMigrationSource(c container, stateful bool) (*migrationSourceWs, error) {
 	ret := migrationSourceWs{migrationFields{container: c}, make(chan bool, 1)}
 
 	var err error
@@ -165,9 +166,10 @@ func NewMigrationSource(c container) (*migrationSourceWs, error) {
 		return nil, err
 	}
 
-	if c.IsRunning() {
-		if err := findCriu("source"); err != nil {
-			return nil, err
+	if stateful && c.IsRunning() {
+		_, err := exec.LookPath("criu")
+		if err != nil {
+			return nil, fmt.Errorf("Unable to perform container live migration. CRIU isn't installed on the source server.")
 		}
 
 		ret.live = true
@@ -290,26 +292,33 @@ func (s *migrationSourceWs) Do(migrateOp *operation) error {
 	criuType := CRIUType_CRIU_RSYNC.Enum()
 	if !s.live {
 		criuType = nil
+	}
 
-		err := s.container.StorageStart()
-		if err != nil {
-			return err
-		}
-
+	// Storage needs to start unconditionally now, since we need to
+	// initialize a new storage interface.
+	ourStart, err := s.container.StorageStart()
+	if err != nil {
+		return err
+	}
+	if ourStart {
 		defer s.container.StorageStop()
 	}
 
 	idmaps := make([]*IDMapType, 0)
 
-	idmapset := s.container.IdmapSet()
+	idmapset, err := s.container.IdmapSet()
+	if err != nil {
+		return err
+	}
+
 	if idmapset != nil {
 		for _, ctnIdmap := range idmapset.Idmap {
 			idmap := IDMapType{
 				Isuid:    proto.Bool(ctnIdmap.Isuid),
 				Isgid:    proto.Bool(ctnIdmap.Isgid),
-				Hostid:   proto.Int(ctnIdmap.Hostid),
-				Nsid:     proto.Int(ctnIdmap.Nsid),
-				Maprange: proto.Int(ctnIdmap.Maprange),
+				Hostid:   proto.Int32(int32(ctnIdmap.Hostid)),
+				Nsid:     proto.Int32(int32(ctnIdmap.Nsid)),
+				Maprange: proto.Int32(int32(ctnIdmap.Maprange)),
 			}
 
 			idmaps = append(idmaps, &idmap)
@@ -594,11 +603,11 @@ func NewMigrationSink(args *MigrationSinkArgs) (*migrationSink, error) {
 		sink.src.live = ok
 	}
 
-	err = findCriu("destination")
+	_, err = exec.LookPath("criu")
 	if sink.push && sink.dest.live && err != nil {
-		return nil, err
+		return nil, fmt.Errorf("Unable to perform container live migration. CRIU isn't installed on the destination server.")
 	} else if sink.src.live && err != nil {
-		return nil, err
+		return nil, fmt.Errorf("Unable to perform container live migration. CRIU isn't installed on the destination server.")
 	}
 
 	return &sink, nil
@@ -757,9 +766,9 @@ func (c *migrationSink) Do(migrateOp *operation) error {
 			e := shared.IdmapEntry{
 				Isuid:    *idmap.Isuid,
 				Isgid:    *idmap.Isgid,
-				Nsid:     int(*idmap.Nsid),
-				Hostid:   int(*idmap.Hostid),
-				Maprange: int(*idmap.Maprange)}
+				Nsid:     int64(*idmap.Nsid),
+				Hostid:   int64(*idmap.Hostid),
+				Maprange: int64(*idmap.Maprange)}
 			srcIdmap.Idmap = shared.Extend(srcIdmap.Idmap, e)
 		}
 
@@ -793,7 +802,8 @@ func (c *migrationSink) Do(migrateOp *operation) error {
 			} else {
 				fsConn = c.src.fsConn
 			}
-			if err := mySink(live, c.src.container, header.Snapshots, fsConn, srcIdmap, migrateOp); err != nil {
+
+			if err := mySink(live, c.src.container, snapshots, fsConn, srcIdmap, migrateOp); err != nil {
 				fsTransfer <- err
 				return
 			}
