@@ -8,7 +8,6 @@ import (
 	"net"
 	"net/http"
 	"os"
-	"os/exec"
 	"strconv"
 	"strings"
 
@@ -16,6 +15,7 @@ import (
 	log "gopkg.in/inconshreveable/log15.v2"
 
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/version"
 )
 
@@ -33,7 +33,7 @@ func networksGet(d *Daemon, r *http.Request) Response {
 	}
 
 	resultString := []string{}
-	resultMap := []shared.NetworkConfig{}
+	resultMap := []api.Network{}
 	for _, iface := range ifs {
 		if recursion == 0 {
 			resultString = append(resultString, fmt.Sprintf("/%s/networks/%s", version.APIVersion, iface))
@@ -54,7 +54,7 @@ func networksGet(d *Daemon, r *http.Request) Response {
 }
 
 func networksPost(d *Daemon, r *http.Request) Response {
-	req := shared.NetworkConfig{}
+	req := api.NetworksPost{}
 
 	// Parse the request
 	err := json.NewDecoder(r.Body).Decode(&req)
@@ -161,18 +161,18 @@ func networkGet(d *Daemon, r *http.Request) Response {
 	return SyncResponseETag(true, &n, etag)
 }
 
-func doNetworkGet(d *Daemon, name string) (shared.NetworkConfig, error) {
+func doNetworkGet(d *Daemon, name string) (api.Network, error) {
 	// Get some information
 	osInfo, _ := net.InterfaceByName(name)
 	_, dbInfo, _ := dbNetworkGet(d.db, name)
 
 	// Sanity check
 	if osInfo == nil && dbInfo == nil {
-		return shared.NetworkConfig{}, os.ErrNotExist
+		return api.Network{}, os.ErrNotExist
 	}
 
 	// Prepare the response
-	n := shared.NetworkConfig{}
+	n := api.Network{}
 	n.Name = name
 	n.UsedBy = []string{}
 	n.Config = map[string]string{}
@@ -180,13 +180,13 @@ func doNetworkGet(d *Daemon, name string) (shared.NetworkConfig, error) {
 	// Look for containers using the interface
 	cts, err := dbContainersList(d.db, cTypeRegular)
 	if err != nil {
-		return shared.NetworkConfig{}, err
+		return api.Network{}, err
 	}
 
 	for _, ct := range cts {
 		c, err := containerLoadByName(d, ct)
 		if err != nil {
-			return shared.NetworkConfig{}, err
+			return api.Network{}, err
 		}
 
 		if networkIsInUse(c, n.Name) {
@@ -204,12 +204,14 @@ func doNetworkGet(d *Daemon, name string) (shared.NetworkConfig, error) {
 		}
 
 		n.Type = "bridge"
+	} else if shared.PathExists(fmt.Sprintf("/proc/net/vlan/%s", n.Name)) {
+		n.Type = "vlan"
 	} else if shared.PathExists(fmt.Sprintf("/sys/class/net/%s/device", n.Name)) {
 		n.Type = "physical"
 	} else if shared.PathExists(fmt.Sprintf("/sys/class/net/%s/bonding", n.Name)) {
 		n.Type = "bond"
 	} else {
-		_, err := exec.Command("ovs-vsctl", "br-exists", n.Name).CombinedOutput()
+		_, err := shared.RunCommand("ovs-vsctl", "br-exists", n.Name)
 		if err == nil {
 			n.Type = "bridge"
 		} else {
@@ -245,7 +247,7 @@ func networkDelete(d *Daemon, r *http.Request) Response {
 
 func networkPost(d *Daemon, r *http.Request) Response {
 	name := mux.Vars(r)["name"]
-	req := shared.NetworkConfig{}
+	req := api.NetworkPost{}
 
 	// Parse the request
 	err := json.NewDecoder(r.Body).Decode(&req)
@@ -305,7 +307,7 @@ func networkPut(d *Daemon, r *http.Request) Response {
 		return PreconditionFailed(err)
 	}
 
-	req := shared.NetworkConfig{}
+	req := api.NetworkPut{}
 	if err := json.NewDecoder(r.Body).Decode(&req); err != nil {
 		return BadRequest(err)
 	}
@@ -330,7 +332,7 @@ func networkPatch(d *Daemon, r *http.Request) Response {
 		return PreconditionFailed(err)
 	}
 
-	req := shared.NetworkConfig{}
+	req := api.NetworkPut{}
 	if err := json.NewDecoder(r.Body).Decode(&req); err != nil {
 		return BadRequest(err)
 	}
@@ -370,7 +372,7 @@ func doNetworkUpdate(d *Daemon, name string, oldConfig map[string]string, newCon
 		return NotFound
 	}
 
-	err = n.Update(shared.NetworkConfig{Config: newConfig})
+	err = n.Update(api.NetworkPut{Config: newConfig})
 	if err != nil {
 		return SmartError(err)
 	}
@@ -522,7 +524,7 @@ func (n *network) Rename(name string) error {
 func (n *network) Start() error {
 	// Create directory
 	if !shared.PathExists(shared.VarPath("networks", n.name)) {
-		err := os.MkdirAll(shared.VarPath("networks", n.name), 0700)
+		err := os.MkdirAll(shared.VarPath("networks", n.name), 0711)
 		if err != nil {
 			return err
 		}
@@ -531,12 +533,12 @@ func (n *network) Start() error {
 	// Create the bridge interface
 	if !n.IsRunning() {
 		if n.config["bridge.driver"] == "openvswitch" {
-			err := shared.RunCommand("ovs-vsctl", "add-br", n.name)
+			_, err := shared.RunCommand("ovs-vsctl", "add-br", n.name)
 			if err != nil {
 				return err
 			}
 		} else {
-			err := shared.RunCommand("ip", "link", "add", n.name, "type", "bridge")
+			_, err := shared.RunCommand("ip", "link", "add", n.name, "type", "bridge")
 			if err != nil {
 				return err
 			}
@@ -568,7 +570,7 @@ func (n *network) Start() error {
 	// Cleanup any existing tunnel device
 	for _, iface := range ifaces {
 		if strings.HasPrefix(iface.Name, fmt.Sprintf("%s-", n.name)) {
-			err = shared.RunCommand("ip", "link", "del", iface.Name)
+			_, err = shared.RunCommand("ip", "link", "del", iface.Name)
 			if err != nil {
 				return err
 			}
@@ -591,7 +593,7 @@ func (n *network) Start() error {
 
 	// Attempt to add a dummy device to the bridge to force the MTU
 	if mtu != "" && n.config["bridge.driver"] != "openvswitch" {
-		err = shared.RunCommand("ip", "link", "add", fmt.Sprintf("%s-mtu", n.name), "mtu", mtu, "type", "dummy")
+		_, err = shared.RunCommand("ip", "link", "add", fmt.Sprintf("%s-mtu", n.name), "mtu", mtu, "type", "dummy")
 		if err == nil {
 			networkAttachInterface(n.name, fmt.Sprintf("%s-mtu", n.name))
 		}
@@ -602,13 +604,13 @@ func (n *network) Start() error {
 		mtu = "1500"
 	}
 
-	err = shared.RunCommand("ip", "link", "set", n.name, "mtu", mtu)
+	_, err = shared.RunCommand("ip", "link", "set", n.name, "mtu", mtu)
 	if err != nil {
 		return err
 	}
 
 	// Bring it up
-	err = shared.RunCommand("ip", "link", "set", n.name, "up")
+	_, err = shared.RunCommand("ip", "link", "set", n.name, "up")
 	if err != nil {
 		return err
 	}
@@ -651,12 +653,12 @@ func (n *network) Start() error {
 	}
 
 	// Flush all IPv4 addresses and routes
-	err = shared.RunCommand("ip", "-4", "addr", "flush", "dev", n.name, "scope", "global")
+	_, err = shared.RunCommand("ip", "-4", "addr", "flush", "dev", n.name, "scope", "global")
 	if err != nil {
 		return err
 	}
 
-	err = shared.RunCommand("ip", "-4", "route", "flush", "dev", n.name, "proto", "static")
+	_, err = shared.RunCommand("ip", "-4", "route", "flush", "dev", n.name, "proto", "static")
 	if err != nil {
 		return err
 	}
@@ -665,12 +667,12 @@ func (n *network) Start() error {
 	if n.config["bridge.mode"] == "fan" || !shared.StringInSlice(n.config["ipv4.address"], []string{"", "none"}) {
 		// Setup basic iptables overrides
 		rules := [][]string{
-			[]string{"ipv4", n.name, "", "INPUT", "-i", n.name, "-p", "udp", "--dport", "67", "-j", "ACCEPT"},
-			[]string{"ipv4", n.name, "", "INPUT", "-i", n.name, "-p", "udp", "--dport", "53", "-j", "ACCEPT"},
-			[]string{"ipv4", n.name, "", "INPUT", "-i", n.name, "-p", "tcp", "--dport", "53", "-j", "ACCEPT"},
-			[]string{"ipv4", n.name, "", "OUTPUT", "-o", n.name, "-p", "udp", "--sport", "67", "-j", "ACCEPT"},
-			[]string{"ipv4", n.name, "", "OUTPUT", "-o", n.name, "-p", "udp", "--sport", "53", "-j", "ACCEPT"},
-			[]string{"ipv4", n.name, "", "OUTPUT", "-o", n.name, "-p", "tcp", "--sport", "53", "-j", "ACCEPT"}}
+			{"ipv4", n.name, "", "INPUT", "-i", n.name, "-p", "udp", "--dport", "67", "-j", "ACCEPT"},
+			{"ipv4", n.name, "", "INPUT", "-i", n.name, "-p", "udp", "--dport", "53", "-j", "ACCEPT"},
+			{"ipv4", n.name, "", "INPUT", "-i", n.name, "-p", "tcp", "--dport", "53", "-j", "ACCEPT"},
+			{"ipv4", n.name, "", "OUTPUT", "-o", n.name, "-p", "udp", "--sport", "67", "-j", "ACCEPT"},
+			{"ipv4", n.name, "", "OUTPUT", "-o", n.name, "-p", "udp", "--sport", "53", "-j", "ACCEPT"},
+			{"ipv4", n.name, "", "OUTPUT", "-o", n.name, "-p", "tcp", "--sport", "53", "-j", "ACCEPT"}}
 
 		for _, rule := range rules {
 			err = networkIptablesPrepend(rule[0], rule[1], rule[2], rule[3], rule[4:]...)
@@ -719,7 +721,7 @@ func (n *network) Start() error {
 	}
 
 	// Start building the dnsmasq command line
-	dnsmasqCmd := []string{"dnsmasq", "-u", "root", "--strict-order", "--bind-interfaces",
+	dnsmasqCmd := []string{"dnsmasq", "--strict-order", "--bind-interfaces",
 		fmt.Sprintf("--pid-file=%s", shared.VarPath("networks", n.name, "dnsmasq.pid")),
 		"--except-interface=lo",
 		fmt.Sprintf("--interface=%s", n.name)}
@@ -739,18 +741,23 @@ func (n *network) Start() error {
 				dnsmasqCmd = append(dnsmasqCmd, []string{"--dhcp-no-override", "--dhcp-authoritative", fmt.Sprintf("--dhcp-leasefile=%s", shared.VarPath("networks", n.name, "dnsmasq.leases")), fmt.Sprintf("--dhcp-hostsfile=%s", shared.VarPath("networks", n.name, "dnsmasq.hosts"))}...)
 			}
 
+			expiry := "1h"
+			if n.config["ipv4.dhcp.expiry"] != "" {
+				expiry = n.config["ipv4.dhcp.expiry"]
+			}
+
 			if n.config["ipv4.dhcp.ranges"] != "" {
 				for _, dhcpRange := range strings.Split(n.config["ipv4.dhcp.ranges"], ",") {
 					dhcpRange = strings.TrimSpace(dhcpRange)
-					dnsmasqCmd = append(dnsmasqCmd, []string{"--dhcp-range", strings.Replace(dhcpRange, "-", ",", -1)}...)
+					dnsmasqCmd = append(dnsmasqCmd, []string{"--dhcp-range", fmt.Sprintf("%s,%s", strings.Replace(dhcpRange, "-", ",", -1), expiry)}...)
 				}
 			} else {
-				dnsmasqCmd = append(dnsmasqCmd, []string{"--dhcp-range", fmt.Sprintf("%s,%s", networkGetIP(subnet, 2).String(), networkGetIP(subnet, -2).String())}...)
+				dnsmasqCmd = append(dnsmasqCmd, []string{"--dhcp-range", fmt.Sprintf("%s,%s,%s", networkGetIP(subnet, 2).String(), networkGetIP(subnet, -2).String(), expiry)}...)
 			}
 		}
 
 		// Add the address
-		err = shared.RunCommand("ip", "-4", "addr", "add", "dev", n.name, n.config["ipv4.address"])
+		_, err = shared.RunCommand("ip", "-4", "addr", "add", "dev", n.name, n.config["ipv4.address"])
 		if err != nil {
 			return err
 		}
@@ -767,7 +774,7 @@ func (n *network) Start() error {
 		if n.config["ipv4.routes"] != "" {
 			for _, route := range strings.Split(n.config["ipv4.routes"], ",") {
 				route = strings.TrimSpace(route)
-				err = shared.RunCommand("ip", "-4", "route", "add", "dev", n.name, route, "proto", "static")
+				_, err = shared.RunCommand("ip", "-4", "route", "add", "dev", n.name, route, "proto", "static")
 				if err != nil {
 					return err
 				}
@@ -787,12 +794,12 @@ func (n *network) Start() error {
 	}
 
 	// Flush all IPv6 addresses and routes
-	err = shared.RunCommand("ip", "-6", "addr", "flush", "dev", n.name, "scope", "global")
+	_, err = shared.RunCommand("ip", "-6", "addr", "flush", "dev", n.name, "scope", "global")
 	if err != nil {
 		return err
 	}
 
-	err = shared.RunCommand("ip", "-6", "route", "flush", "dev", n.name, "proto", "static")
+	_, err = shared.RunCommand("ip", "-6", "route", "flush", "dev", n.name, "proto", "static")
 	if err != nil {
 		return err
 	}
@@ -818,14 +825,19 @@ func (n *network) Start() error {
 				dnsmasqCmd = append(dnsmasqCmd, []string{"--dhcp-no-override", "--dhcp-authoritative", fmt.Sprintf("--dhcp-leasefile=%s", shared.VarPath("networks", n.name, "dnsmasq.leases")), fmt.Sprintf("--dhcp-hostsfile=%s", shared.VarPath("networks", n.name, "dnsmasq.hosts"))}...)
 			}
 
+			expiry := "1h"
+			if n.config["ipv6.dhcp.expiry"] != "" {
+				expiry = n.config["ipv6.dhcp.expiry"]
+			}
+
 			if shared.IsTrue(n.config["ipv6.dhcp.stateful"]) {
 				if n.config["ipv6.dhcp.ranges"] != "" {
 					for _, dhcpRange := range strings.Split(n.config["ipv6.dhcp.ranges"], ",") {
 						dhcpRange = strings.TrimSpace(dhcpRange)
-						dnsmasqCmd = append(dnsmasqCmd, []string{"--dhcp-range", fmt.Sprintf("%s", strings.Replace(dhcpRange, "-", ",", -1))}...)
+						dnsmasqCmd = append(dnsmasqCmd, []string{"--dhcp-range", fmt.Sprintf("%s,%s", strings.Replace(dhcpRange, "-", ",", -1), expiry)}...)
 					}
 				} else {
-					dnsmasqCmd = append(dnsmasqCmd, []string{"--dhcp-range", fmt.Sprintf("%s,%s", networkGetIP(subnet, 2), networkGetIP(subnet, -1))}...)
+					dnsmasqCmd = append(dnsmasqCmd, []string{"--dhcp-range", fmt.Sprintf("%s,%s,%s", networkGetIP(subnet, 2), networkGetIP(subnet, -1), expiry)}...)
 				}
 			} else {
 				dnsmasqCmd = append(dnsmasqCmd, []string{"--dhcp-range", fmt.Sprintf("::,constructor:%s,ra-stateless,ra-names", n.name)}...)
@@ -836,12 +848,12 @@ func (n *network) Start() error {
 
 		// Setup basic iptables overrides
 		rules := [][]string{
-			[]string{"ipv6", n.name, "", "INPUT", "-i", n.name, "-p", "udp", "--dport", "546", "-j", "ACCEPT"},
-			[]string{"ipv6", n.name, "", "INPUT", "-i", n.name, "-p", "udp", "--dport", "53", "-j", "ACCEPT"},
-			[]string{"ipv6", n.name, "", "INPUT", "-i", n.name, "-p", "tcp", "--dport", "53", "-j", "ACCEPT"},
-			[]string{"ipv6", n.name, "", "OUTPUT", "-o", n.name, "-p", "udp", "--sport", "546", "-j", "ACCEPT"},
-			[]string{"ipv6", n.name, "", "OUTPUT", "-o", n.name, "-p", "udp", "--sport", "53", "-j", "ACCEPT"},
-			[]string{"ipv6", n.name, "", "OUTPUT", "-o", n.name, "-p", "tcp", "--sport", "53", "-j", "ACCEPT"}}
+			{"ipv6", n.name, "", "INPUT", "-i", n.name, "-p", "udp", "--dport", "546", "-j", "ACCEPT"},
+			{"ipv6", n.name, "", "INPUT", "-i", n.name, "-p", "udp", "--dport", "53", "-j", "ACCEPT"},
+			{"ipv6", n.name, "", "INPUT", "-i", n.name, "-p", "tcp", "--dport", "53", "-j", "ACCEPT"},
+			{"ipv6", n.name, "", "OUTPUT", "-o", n.name, "-p", "udp", "--sport", "546", "-j", "ACCEPT"},
+			{"ipv6", n.name, "", "OUTPUT", "-o", n.name, "-p", "udp", "--sport", "53", "-j", "ACCEPT"},
+			{"ipv6", n.name, "", "OUTPUT", "-o", n.name, "-p", "tcp", "--sport", "53", "-j", "ACCEPT"}}
 
 		for _, rule := range rules {
 			err = networkIptablesPrepend(rule[0], rule[1], rule[2], rule[3], rule[4:]...)
@@ -905,7 +917,7 @@ func (n *network) Start() error {
 		}
 
 		// Add the address
-		err = shared.RunCommand("ip", "-6", "addr", "add", "dev", n.name, n.config["ipv6.address"])
+		_, err = shared.RunCommand("ip", "-6", "addr", "add", "dev", n.name, n.config["ipv6.address"])
 		if err != nil {
 			return err
 		}
@@ -922,7 +934,7 @@ func (n *network) Start() error {
 		if n.config["ipv6.routes"] != "" {
 			for _, route := range strings.Split(n.config["ipv6.routes"], ",") {
 				route = strings.TrimSpace(route)
-				err = shared.RunCommand("ip", "-6", "route", "add", "dev", n.name, route, "proto", "static")
+				_, err = shared.RunCommand("ip", "-6", "route", "add", "dev", n.name, route, "proto", "static")
 				if err != nil {
 					return err
 				}
@@ -970,7 +982,7 @@ func (n *network) Start() error {
 		}
 
 		// Add the address
-		err = shared.RunCommand("ip", "-4", "addr", "add", "dev", n.name, fanAddress)
+		_, err = shared.RunCommand("ip", "-4", "addr", "add", "dev", n.name, fanAddress)
 		if err != nil {
 			return err
 		}
@@ -985,12 +997,12 @@ func (n *network) Start() error {
 
 		// Setup the tunnel
 		if n.config["fan.type"] == "ipip" {
-			err = shared.RunCommand("ip", "-4", "route", "flush", "dev", "tunl0")
+			_, err = shared.RunCommand("ip", "-4", "route", "flush", "dev", "tunl0")
 			if err != nil {
 				return err
 			}
 
-			err = shared.RunCommand("ip", "link", "set", "tunl0", "up")
+			_, err = shared.RunCommand("ip", "link", "set", "tunl0", "up")
 			if err != nil {
 				return err
 			}
@@ -998,14 +1010,14 @@ func (n *network) Start() error {
 			// Fails if the map is already set
 			shared.RunCommand("ip", "link", "change", "tunl0", "type", "ipip", "fan-map", fmt.Sprintf("%s:%s", overlay, underlay))
 
-			err = shared.RunCommand("ip", "route", "add", overlay, "dev", "tunl0", "src", addr[0])
+			_, err = shared.RunCommand("ip", "route", "add", overlay, "dev", "tunl0", "src", addr[0])
 			if err != nil {
 				return err
 			}
 		} else {
 			vxlanID := fmt.Sprintf("%d", binary.BigEndian.Uint32(overlaySubnet.IP.To4())>>8)
 
-			err = shared.RunCommand("ip", "link", "add", tunName, "type", "vxlan", "id", vxlanID, "dev", devName, "dstport", "0", "local", devAddr, "fan-map", fmt.Sprintf("%s:%s", overlay, underlay))
+			_, err = shared.RunCommand("ip", "link", "add", tunName, "type", "vxlan", "id", vxlanID, "dev", devName, "dstport", "0", "local", devAddr, "fan-map", fmt.Sprintf("%s:%s", overlay, underlay))
 			if err != nil {
 				return err
 			}
@@ -1015,12 +1027,12 @@ func (n *network) Start() error {
 				return err
 			}
 
-			err = shared.RunCommand("ip", "link", "set", tunName, "mtu", mtu, "up")
+			_, err = shared.RunCommand("ip", "link", "set", tunName, "mtu", mtu, "up")
 			if err != nil {
 				return err
 			}
 
-			err = shared.RunCommand("ip", "link", "set", n.name, "up")
+			_, err = shared.RunCommand("ip", "link", "set", n.name, "up")
 			if err != nil {
 				return err
 			}
@@ -1092,7 +1104,7 @@ func (n *network) Start() error {
 		}
 
 		// Create the interface
-		err = shared.RunCommand(cmd[0], cmd[1:]...)
+		_, err = shared.RunCommand(cmd[0], cmd[1:]...)
 		if err != nil {
 			return err
 		}
@@ -1103,12 +1115,12 @@ func (n *network) Start() error {
 			return err
 		}
 
-		err = shared.RunCommand("ip", "link", "set", tunName, "mtu", mtu, "up")
+		_, err = shared.RunCommand("ip", "link", "set", tunName, "mtu", mtu, "up")
 		if err != nil {
 			return err
 		}
 
-		err = shared.RunCommand("ip", "link", "set", n.name, "up")
+		_, err = shared.RunCommand("ip", "link", "set", n.name, "up")
 		if err != nil {
 			return err
 		}
@@ -1122,12 +1134,12 @@ func (n *network) Start() error {
 
 	// Configure dnsmasq
 	if n.config["bridge.mode"] == "fan" || !shared.StringInSlice(n.config["ipv4.address"], []string{"", "none"}) || !shared.StringInSlice(n.config["ipv6.address"], []string{"", "none"}) {
+		// Setup the dnsmasq domain
 		dnsDomain := n.config["dns.domain"]
 		if dnsDomain == "" {
 			dnsDomain = "lxd"
 		}
 
-		// Setup the dnsmasq domain
 		if n.config["dns.mode"] != "none" {
 			dnsmasqCmd = append(dnsmasqCmd, []string{"-s", dnsDomain, "-S", fmt.Sprintf("/%s/", dnsDomain)}...)
 		}
@@ -1143,16 +1155,27 @@ func (n *network) Start() error {
 
 		// Create DHCP hosts file
 		if !shared.PathExists(shared.VarPath("networks", n.name, "dnsmasq.hosts")) {
-			err = ioutil.WriteFile(shared.VarPath("networks", n.name, "dnsmasq.hosts"), []byte(""), 0)
+			err = ioutil.WriteFile(shared.VarPath("networks", n.name, "dnsmasq.hosts"), []byte(""), 0644)
 			if err != nil {
 				return err
 			}
 		}
 
-		// Start dnsmasq (occasionaly races, try a few times)
-		output, err := tryExec(dnsmasqCmd[0], dnsmasqCmd[1:]...)
+		// Attempt to drop privileges
+		for _, user := range []string{"lxd", "nobody"} {
+			_, err := shared.UserId(user)
+			if err != nil {
+				continue
+			}
+
+			dnsmasqCmd = append(dnsmasqCmd, []string{"-u", user}...)
+			break
+		}
+
+		// Start dnsmasq (occasionally races, try a few times)
+		output, err := shared.TryRunCommand(dnsmasqCmd[0], dnsmasqCmd[1:]...)
 		if err != nil {
-			return fmt.Errorf("Failed to run: %s: %s", strings.Join(dnsmasqCmd, " "), strings.TrimSpace(string(output)))
+			return fmt.Errorf("Failed to run: %s: %s", strings.Join(dnsmasqCmd, " "), strings.TrimSpace(output))
 		}
 
 		// Update the static leases
@@ -1172,12 +1195,12 @@ func (n *network) Stop() error {
 
 	// Destroy the bridge interface
 	if n.config["bridge.driver"] == "openvswitch" {
-		err := shared.RunCommand("ovs-vsctl", "del-br", n.name)
+		_, err := shared.RunCommand("ovs-vsctl", "del-br", n.name)
 		if err != nil {
 			return err
 		}
 	} else {
-		err := shared.RunCommand("ip", "link", "del", n.name)
+		_, err := shared.RunCommand("ip", "link", "del", n.name)
 		if err != nil {
 			return err
 		}
@@ -1224,7 +1247,7 @@ func (n *network) Stop() error {
 	// Cleanup any existing tunnel device
 	for _, iface := range ifaces {
 		if strings.HasPrefix(iface.Name, fmt.Sprintf("%s-", n.name)) {
-			err = shared.RunCommand("ip", "link", "del", iface.Name)
+			_, err = shared.RunCommand("ip", "link", "del", iface.Name)
 			if err != nil {
 				return err
 			}
@@ -1234,7 +1257,7 @@ func (n *network) Stop() error {
 	return nil
 }
 
-func (n *network) Update(newNetwork shared.NetworkConfig) error {
+func (n *network) Update(newNetwork api.NetworkPut) error {
 	err := networkFillAuto(newNetwork.Config)
 	if err != nil {
 		return err
@@ -1262,7 +1285,7 @@ func (n *network) Update(newNetwork shared.NetworkConfig) error {
 	// Diff the configurations
 	changedConfig := []string{}
 	userOnly := true
-	for key, _ := range oldConfig {
+	for key := range oldConfig {
 		if oldConfig[key] != newConfig[key] {
 			if !strings.HasPrefix(key, "user.") {
 				userOnly = false
@@ -1274,7 +1297,7 @@ func (n *network) Update(newNetwork shared.NetworkConfig) error {
 		}
 	}
 
-	for key, _ := range newConfig {
+	for key := range newConfig {
 		if oldConfig[key] != newConfig[key] {
 			if !strings.HasPrefix(key, "user.") {
 				userOnly = false
