@@ -6,26 +6,19 @@ import (
 	"fmt"
 	"io/ioutil"
 	"net/http"
-	"reflect"
 	"strings"
 
 	"github.com/gorilla/mux"
 	_ "github.com/mattn/go-sqlite3"
 
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/version"
 
 	log "gopkg.in/inconshreveable/log15.v2"
 )
 
 /* This is used for both profiles post and profile put */
-type profilesPostReq struct {
-	Name        string            `json:"name"`
-	Config      map[string]string `json:"config"`
-	Description string            `json:"description"`
-	Devices     shared.Devices    `json:"devices"`
-}
-
 func profilesGet(d *Daemon, r *http.Request) Response {
 	results, err := dbProfiles(d.db)
 	if err != nil {
@@ -35,7 +28,7 @@ func profilesGet(d *Daemon, r *http.Request) Response {
 	recursion := d.isRecursionRequest(r)
 
 	resultString := make([]string, len(results))
-	resultMap := make([]*shared.ProfileConfig, len(results))
+	resultMap := make([]*api.Profile, len(results))
 	i := 0
 	for _, name := range results {
 		if !recursion {
@@ -60,7 +53,7 @@ func profilesGet(d *Daemon, r *http.Request) Response {
 }
 
 func profilesPost(d *Daemon, r *http.Request) Response {
-	req := profilesPostReq{}
+	req := api.ProfilesPost{}
 	if err := json.NewDecoder(r.Body).Decode(&req); err != nil {
 		return BadRequest(err)
 	}
@@ -108,7 +101,7 @@ var profilesCmd = Command{
 	get:  profilesGet,
 	post: profilesPost}
 
-func doProfileGet(d *Daemon, name string) (*shared.ProfileConfig, error) {
+func doProfileGet(d *Daemon, name string) (*api.Profile, error) {
 	_, profile, err := dbProfileGet(d.db, name)
 	if err != nil {
 		return nil, err
@@ -173,7 +166,7 @@ func profilePut(d *Daemon, r *http.Request) Response {
 		return PreconditionFailed(err)
 	}
 
-	req := profilesPostReq{}
+	req := api.ProfilePut{}
 	if err := json.NewDecoder(r.Body).Decode(&req); err != nil {
 		return BadRequest(err)
 	}
@@ -208,7 +201,7 @@ func profilePatch(d *Daemon, r *http.Request) Response {
 		return BadRequest(err)
 	}
 
-	req := profilesPostReq{}
+	req := api.ProfilePut{}
 	if err := json.NewDecoder(rdr2).Decode(&req); err != nil {
 		return BadRequest(err)
 	}
@@ -246,99 +239,11 @@ func profilePatch(d *Daemon, r *http.Request) Response {
 	return doProfileUpdate(d, name, id, profile, req)
 }
 
-func doProfileUpdate(d *Daemon, name string, id int64, profile *shared.ProfileConfig, req profilesPostReq) Response {
-	// Sanity checks
-	err := containerValidConfig(d, req.Config, true, false)
-	if err != nil {
-		return BadRequest(err)
-	}
-
-	err = containerValidDevices(req.Devices, true, false)
-	if err != nil {
-		return BadRequest(err)
-	}
-
-	// Get the container list
-	containers := getContainersWithProfile(d, name)
-
-	// Update the database
-	tx, err := dbBegin(d.db)
-	if err != nil {
-		return InternalError(err)
-	}
-
-	if profile.Description != req.Description {
-		err = dbProfileDescriptionUpdate(tx, id, req.Description)
-		if err != nil {
-			tx.Rollback()
-			return InternalError(err)
-		}
-	}
-
-	// Optimize for description-only changes
-	if reflect.DeepEqual(profile.Config, req.Config) && reflect.DeepEqual(profile.Devices, req.Devices) {
-		err = txCommit(tx)
-		if err != nil {
-			return InternalError(err)
-		}
-
-		return EmptySyncResponse
-	}
-
-	err = dbProfileConfigClear(tx, id)
-	if err != nil {
-		tx.Rollback()
-		return InternalError(err)
-	}
-
-	err = dbProfileConfigAdd(tx, id, req.Config)
-	if err != nil {
-		tx.Rollback()
-		return SmartError(err)
-	}
-
-	err = dbDevicesAdd(tx, "profile", id, req.Devices)
-	if err != nil {
-		tx.Rollback()
-		return SmartError(err)
-	}
-
-	err = txCommit(tx)
-	if err != nil {
-		return InternalError(err)
-	}
-
-	// Update all the containers using the profile. Must be done after txCommit due to DB lock.
-	failures := map[string]error{}
-	for _, c := range containers {
-		err = c.Update(containerArgs{
-			Architecture: c.Architecture(),
-			Ephemeral:    c.IsEphemeral(),
-			Config:       c.LocalConfig(),
-			Devices:      c.LocalDevices(),
-			Profiles:     c.Profiles()}, true)
-
-		if err != nil {
-			failures[c.Name()] = err
-		}
-	}
-
-	if len(failures) != 0 {
-		msg := "The following containers failed to update (profile change still saved):\n"
-		for cname, err := range failures {
-			msg += fmt.Sprintf(" - %s: %s\n", cname, err)
-		}
-		return InternalError(fmt.Errorf("%s", msg))
-	}
-
-	return EmptySyncResponse
-}
-
 // The handler for the post operation.
 func profilePost(d *Daemon, r *http.Request) Response {
 	name := mux.Vars(r)["name"]
 
-	req := profilesPostReq{}
+	req := api.ProfilePost{}
 	if err := json.NewDecoder(r.Body).Decode(&req); err != nil {
 		return BadRequest(err)
 	}
