@@ -5,16 +5,26 @@ import (
 	"io"
 	"io/ioutil"
 	"net"
+	"os"
 	"os/exec"
 
 	"github.com/gorilla/websocket"
-	"github.com/pborman/uuid"
 
 	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/logger"
 )
 
-func rsyncSendSetup(name string, path string) (*exec.Cmd, net.Conn, io.ReadCloser, error) {
+func rsyncSendSetup(path string) (*exec.Cmd, net.Conn, io.ReadCloser, error) {
+	/*
+	 * It's sort of unfortunate, but there's no library call to get a
+	 * temporary name, so we get the file and close it and use its name.
+	 */
+	f, err := ioutil.TempFile("", "lxd_rsync_")
+	if err != nil {
+		return nil, nil, nil, err
+	}
+	f.Close()
+	os.Remove(f.Name())
+
 	/*
 	 * The way rsync works, it invokes a subprocess that does the actual
 	 * talking (given to it by a -E argument). Since there isn't an easy
@@ -29,14 +39,7 @@ func rsyncSendSetup(name string, path string) (*exec.Cmd, net.Conn, io.ReadClose
 	 * stdin/stdout, but that also seemed messy. In any case, this seems to
 	 * work just fine.
 	 */
-	auds := fmt.Sprintf("@lxd/%s", uuid.NewRandom().String())
-	// We simply copy a part of the uuid if it's longer than the allowed
-	// maximum. That should be safe enough for our purposes.
-	if len(auds) > shared.ABSTRACT_UNIX_SOCK_LEN-1 {
-		auds = auds[:shared.ABSTRACT_UNIX_SOCK_LEN-1]
-	}
-
-	l, err := net.Listen("unix", auds)
+	l, err := net.Listen("unix", f.Name())
 	if err != nil {
 		return nil, nil, nil, err
 	}
@@ -52,14 +55,13 @@ func rsyncSendSetup(name string, path string) (*exec.Cmd, net.Conn, io.ReadClose
 	 * command (i.e. the command to run on --server). However, we're
 	 * hardcoding that at the other end, so we can just ignore it.
 	 */
-	rsyncCmd := fmt.Sprintf("sh -c \"%s netcat %s %s\"", execPath, auds, name)
+	rsyncCmd := fmt.Sprintf("sh -c \"%s netcat %s\"", execPath, f.Name())
 	cmd := exec.Command(
 		"rsync",
 		"-arvP",
 		"--devices",
 		"--numeric-ids",
 		"--partial",
-		"--sparse",
 		path,
 		"localhost:/tmp/foo",
 		"-e",
@@ -87,8 +89,8 @@ func rsyncSendSetup(name string, path string) (*exec.Cmd, net.Conn, io.ReadClose
 
 // RsyncSend sets up the sending half of an rsync, to recursively send the
 // directory pointed to by path over the websocket.
-func RsyncSend(name string, path string, conn *websocket.Conn) error {
-	cmd, dataSocket, stderr, err := rsyncSendSetup(name, path)
+func RsyncSend(path string, conn *websocket.Conn, readWrapper func(io.ReadCloser) io.ReadCloser) error {
+	cmd, dataSocket, stderr, err := rsyncSendSetup(path)
 	if err != nil {
 		return err
 	}
@@ -97,7 +99,12 @@ func RsyncSend(name string, path string, conn *websocket.Conn) error {
 		defer dataSocket.Close()
 	}
 
-	readDone, writeDone := shared.WebsocketMirror(conn, dataSocket, dataSocket, nil, nil)
+	readPipe := io.ReadCloser(dataSocket)
+	if readWrapper != nil {
+		readPipe = readWrapper(dataSocket)
+	}
+
+	readDone, writeDone := shared.WebsocketMirror(conn, dataSocket, readPipe, nil, nil)
 
 	output, err := ioutil.ReadAll(stderr)
 	if err != nil {
@@ -108,7 +115,7 @@ func RsyncSend(name string, path string, conn *websocket.Conn) error {
 
 	err = cmd.Wait()
 	if err != nil {
-		logger.Errorf("Rsync send failed: %s: %s: %s", path, err, string(output))
+		shared.LogErrorf("Rsync send failed: %s: %s: %s", path, err, string(output))
 	}
 
 	<-readDone
@@ -120,14 +127,13 @@ func RsyncSend(name string, path string, conn *websocket.Conn) error {
 // RsyncRecv sets up the receiving half of the websocket to rsync (the other
 // half set up by RsyncSend), putting the contents in the directory specified
 // by path.
-func RsyncRecv(path string, conn *websocket.Conn) error {
+func RsyncRecv(path string, conn *websocket.Conn, writeWrapper func(io.WriteCloser) io.WriteCloser) error {
 	cmd := exec.Command("rsync",
 		"--server",
 		"-vlogDtpre.iLsfx",
 		"--numeric-ids",
 		"--devices",
 		"--partial",
-		"--sparse",
 		".",
 		path)
 
@@ -150,7 +156,12 @@ func RsyncRecv(path string, conn *websocket.Conn) error {
 		return err
 	}
 
-	readDone, writeDone := shared.WebsocketMirror(conn, stdin, stdout, nil, nil)
+	writePipe := io.WriteCloser(stdin)
+	if writeWrapper != nil {
+		writePipe = writeWrapper(stdin)
+	}
+
+	readDone, writeDone := shared.WebsocketMirror(conn, writePipe, stdout, nil, nil)
 	output, err := ioutil.ReadAll(stderr)
 	if err != nil {
 		cmd.Process.Kill()
@@ -160,7 +171,7 @@ func RsyncRecv(path string, conn *websocket.Conn) error {
 
 	err = cmd.Wait()
 	if err != nil {
-		logger.Errorf("Rsync receive failed: %s: %s: %s", path, err, string(output))
+		shared.LogErrorf("Rsync receive failed: %s: %s: %s", path, err, string(output))
 	}
 
 	<-readDone
