@@ -10,8 +10,6 @@ type ContainersPost struct {
 
 	Name   string          `json:"name" yaml:"name"`
 	Source ContainerSource `json:"source" yaml:"source"`
-
-	InstanceType string `json:"instance_type" yaml:"instance_type"`
 }
 
 // ContainerPost represents the fields required to rename/move a LXD container
@@ -24,21 +22,6 @@ type ContainerPost struct {
 
 	// API extension: container_stateless_copy
 	Live bool `json:"live" yaml:"live"`
-
-	// API extension: container_only_migration
-	ContainerOnly bool `json:"container_only" yaml:"container_only"`
-
-	// API extension: container_push_target
-	Target *ContainerPostTarget `json:"target" yaml:"target"`
-}
-
-// ContainerPostTarget represents the migration target host and operation
-//
-// API extension: container_push_target
-type ContainerPostTarget struct {
-	Certificate string            `json:"certificate" yaml:"certificate"`
-	Operation   string            `json:"operation,omitempty" yaml:"operation,omitempty"`
-	Websockets  map[string]string `json:"secrets,omitempty" yaml:"secrets,omitempty"`
 }
 
 // ContainerPut represents the modifiable fields of a LXD container
@@ -48,13 +31,7 @@ type ContainerPut struct {
 	Devices      map[string]map[string]string `json:"devices" yaml:"devices"`
 	Ephemeral    bool                         `json:"ephemeral" yaml:"ephemeral"`
 	Profiles     []string                     `json:"profiles" yaml:"profiles"`
-
-	// For snapshot restore
-	Restore  string `json:"restore,omitempty" yaml:"restore,omitempty"`
-	Stateful bool   `json:"stateful" yaml:"stateful"`
-
-	// API extension: entity_description
-	Description string `json:"description" yaml:"description"`
+	Restore      string                       `json:"restore,omitempty" yaml:"restore,omitempty"`
 }
 
 // Container represents a LXD container
@@ -65,6 +42,7 @@ type Container struct {
 	ExpandedConfig  map[string]string            `json:"expanded_config" yaml:"expanded_config"`
 	ExpandedDevices map[string]map[string]string `json:"expanded_devices" yaml:"expanded_devices"`
 	Name            string                       `json:"name" yaml:"name"`
+	Stateful        bool                         `json:"stateful" yaml:"stateful"`
 	Status          string                       `json:"status" yaml:"status"`
 	StatusCode      StatusCode                   `json:"status_code" yaml:"status_code"`
 
@@ -110,12 +88,9 @@ type ContainerSource struct {
 	Operation  string            `json:"operation,omitempty" yaml:"operation,omitempty"`
 	Websockets map[string]string `json:"secrets,omitempty" yaml:"secrets,omitempty"`
 
-	// For "copy" type
-	Source string `json:"source,omitempty" yaml:"source,omitempty"`
-
 	// API extension: container_push
 	Live bool `json:"live,omitempty" yaml:"live,omitempty"`
 
-	// API extension: container_only_migration
-	ContainerOnly bool `json:"container_only,omitempty" yaml:"container_only,omitempty"`
+	// For "copy" type
+	Source string `json:"source,omitempty" yaml:"source,omitempty"`
 }
