@@ -4,7 +4,6 @@ import (
 	"time"
 )
 
-// ProgressTracker provides the stream information needed for tracking
 type ProgressTracker struct {
 	Length  int64
 	Handler func(int64, int64)
@@ -15,7 +14,7 @@ type ProgressTracker struct {
 	last       *time.Time
 }
 
-func (pt *ProgressTracker) update(n int) {
+func (pt *ProgressTracker) Update(n int) {
 	// Skip the rest if no handler attached
 	if pt.Handler == nil {
 		return
