@@ -10,15 +10,17 @@ import (
 	"os"
 	"path/filepath"
 	"sort"
+	"strconv"
 	"strings"
 	"time"
 
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/ioprogress"
 	"github.com/lxc/lxd/shared/osarch"
 )
 
-type ssSortImage []shared.ImageInfo
+type ssSortImage []api.Image
 
 func (a ssSortImage) Len() int {
 	return len(a)
@@ -31,15 +33,19 @@ func (a ssSortImage) Swap(i, j int) {
 func (a ssSortImage) Less(i, j int) bool {
 	if a[i].Properties["os"] == a[j].Properties["os"] {
 		if a[i].Properties["release"] == a[j].Properties["release"] {
-			if a[i].CreationDate.UTC().Unix() == 0 {
+			if !shared.TimeIsSet(a[i].CreatedAt) {
 				return true
 			}
 
-			if a[j].CreationDate.UTC().Unix() == 0 {
+			if !shared.TimeIsSet(a[j].CreatedAt) {
 				return false
 			}
 
-			return a[i].CreationDate.UTC().Unix() > a[j].CreationDate.UTC().Unix()
+			if a[i].CreatedAt == a[j].CreatedAt {
+				return a[i].Properties["serial"] > a[j].Properties["serial"]
+			}
+
+			return a[i].CreatedAt.UTC().Unix() > a[j].CreatedAt.UTC().Unix()
 		}
 
 		if a[i].Properties["release"] == "" {
@@ -76,10 +82,10 @@ type SimpleStreamsManifest struct {
 	Products map[string]SimpleStreamsManifestProduct `json:"products"`
 }
 
-func (s *SimpleStreamsManifest) ToLXD() ([]shared.ImageInfo, map[string][][]string) {
+func (s *SimpleStreamsManifest) ToLXD() ([]api.Image, map[string][][]string) {
 	downloads := map[string][][]string{}
 
-	images := []shared.ImageInfo{}
+	images := []api.Image{}
 	nameLayout := "20060102"
 	eolLayout := "2006-01-02"
 
@@ -106,58 +112,64 @@ func (s *SimpleStreamsManifest) ToLXD() ([]shared.ImageInfo, map[string][][]stri
 				continue
 			}
 
-			size := int64(0)
-			filename := ""
-			fingerprint := ""
+			var meta SimpleStreamsManifestProductVersionItem
+			var rootTar SimpleStreamsManifestProductVersionItem
+			var rootSquash SimpleStreamsManifestProductVersionItem
 
-			metaPath := ""
-			metaHash := ""
-			rootfsPath := ""
-			rootfsHash := ""
-
-			found := 0
 			for _, item := range version.Items {
 				// Skip the files we don't care about
 				if !shared.StringInSlice(item.FileType, []string{"root.tar.xz", "lxd.tar.xz", "squashfs"}) {
 					continue
 				}
-				found += 1
-
-				if fingerprint == "" {
-					if item.LXDHashSha256SquashFs != "" {
-						fingerprint = item.LXDHashSha256SquashFs
-					} else if item.LXDHashSha256RootXz != "" {
-						fingerprint = item.LXDHashSha256RootXz
-					} else if item.LXDHashSha256 != "" {
-						fingerprint = item.LXDHashSha256
-					}
-				}
 
 				if item.FileType == "lxd.tar.xz" {
-					fields := strings.Split(item.Path, "/")
-					filename = fields[len(fields)-1]
-					metaPath = item.Path
-					metaHash = item.HashSha256
-
-					size += item.Size
-				}
-
-				if rootfsPath == "" || rootfsHash == "" {
-					if item.FileType == "squashfs" {
-						rootfsPath = item.Path
-						rootfsHash = item.HashSha256
-					}
-
-					if item.FileType == "root.tar.xz" {
-						rootfsPath = item.Path
-						rootfsHash = item.HashSha256
-					}
-
-					size += item.Size
+					meta = item
+				} else if item.FileType == "squashfs" {
+					rootSquash = item
+				} else if item.FileType == "root.tar.xz" {
+					rootTar = item
 				}
 			}
 
-			if found < 2 || size == 0 || filename == "" || fingerprint == "" {
+			if meta.FileType == "" || (rootTar.FileType == "" && rootSquash.FileType == "") {
+				// Invalid image
+				continue
+			}
+
+			metaPath := meta.Path
+			metaHash := meta.HashSha256
+			metaSize := meta.Size
+			rootfsPath := ""
+			rootfsHash := ""
+			rootfsSize := int64(0)
+			fields := strings.Split(meta.Path, "/")
+			filename := fields[len(fields)-1]
+			size := meta.Size
+			fingerprint := ""
+
+			if rootSquash.FileType != "" {
+				if meta.LXDHashSha256SquashFs != "" {
+					fingerprint = meta.LXDHashSha256SquashFs
+				} else {
+					fingerprint = meta.LXDHashSha256
+				}
+				size += rootSquash.Size
+				rootfsPath = rootSquash.Path
+				rootfsHash = rootSquash.HashSha256
+				rootfsSize = rootSquash.Size
+			} else {
+				if meta.LXDHashSha256RootXz != "" {
+					fingerprint = meta.LXDHashSha256RootXz
+				} else {
+					fingerprint = meta.LXDHashSha256
+				}
+				size += rootTar.Size
+				rootfsPath = rootTar.Path
+				rootfsHash = rootTar.HashSha256
+				rootfsSize = rootTar.Size
+			}
+
+			if size == 0 || filename == "" || fingerprint == "" {
 				// Invalid image
 				continue
 			}
@@ -169,12 +181,12 @@ func (s *SimpleStreamsManifest) ToLXD() ([]shared.ImageInfo, map[string][][]stri
 			}
 			description = fmt.Sprintf("%s (%s)", description, name)
 
-			image := shared.ImageInfo{}
+			image := api.Image{}
 			image.Architecture = architectureName
 			image.Public = true
 			image.Size = size
-			image.CreationDate = creationDate
-			image.UploadDate = creationDate
+			image.CreatedAt = creationDate
+			image.UploadedAt = creationDate
 			image.Filename = filename
 			image.Fingerprint = fingerprint
 			image.Properties = map[string]string{
@@ -189,9 +201,9 @@ func (s *SimpleStreamsManifest) ToLXD() ([]shared.ImageInfo, map[string][][]stri
 
 			// Add the provided aliases
 			if product.Aliases != "" {
-				image.Aliases = []shared.ImageAlias{}
+				image.Aliases = []api.ImageAlias{}
 				for _, entry := range strings.Split(product.Aliases, ",") {
-					image.Aliases = append(image.Aliases, shared.ImageAlias{Name: entry})
+					image.Aliases = append(image.Aliases, api.ImageAlias{Name: entry})
 				}
 			}
 
@@ -203,15 +215,17 @@ func (s *SimpleStreamsManifest) ToLXD() ([]shared.ImageInfo, map[string][][]stri
 			}
 
 			// Attempt to parse the EOL
-			image.ExpiryDate = time.Unix(0, 0).UTC()
+			image.ExpiresAt = time.Unix(0, 0).UTC()
 			if product.SupportedEOL != "" {
 				eolDate, err := time.Parse(eolLayout, product.SupportedEOL)
 				if err == nil {
-					image.ExpiryDate = eolDate
+					image.ExpiresAt = eolDate
 				}
 			}
 
-			downloads[fingerprint] = [][]string{[]string{metaPath, metaHash, "meta"}, []string{rootfsPath, rootfsHash, "root"}}
+			downloads[fingerprint] = [][]string{
+				{metaPath, metaHash, "meta", fmt.Sprintf("%d", metaSize)},
+				{rootfsPath, rootfsHash, "root", fmt.Sprintf("%d", rootfsSize)}}
 			images = append(images, image)
 		}
 	}
@@ -262,6 +276,12 @@ type SimpleStreamsIndexStream struct {
 	Products []string `json:"products"`
 }
 
+type SimpleStreamsFile struct {
+	Path   string
+	Sha256 string
+	Size   int64
+}
+
 func NewClient(url string, httpClient http.Client, useragent string) *SimpleStreams {
 	return &SimpleStreams{
 		http:           &httpClient,
@@ -278,8 +298,8 @@ type SimpleStreams struct {
 
 	cachedIndex    *SimpleStreamsIndex
 	cachedManifest map[string]*SimpleStreamsManifest
-	cachedImages   []shared.ImageInfo
-	cachedAliases  map[string]*shared.ImageAliasesEntry
+	cachedImages   []api.Image
+	cachedAliases  map[string]*api.ImageAliasesEntry
 }
 
 func (s *SimpleStreams) parseIndex() (*SimpleStreamsIndex, error) {
@@ -287,7 +307,8 @@ func (s *SimpleStreams) parseIndex() (*SimpleStreamsIndex, error) {
 		return s.cachedIndex, nil
 	}
 
-	req, err := http.NewRequest("GET", fmt.Sprintf("%s/streams/v1/index.json", s.url), nil)
+	url := fmt.Sprintf("%s/streams/v1/index.json", s.url)
+	req, err := http.NewRequest("GET", url, nil)
 	if err != nil {
 		return nil, err
 	}
@@ -302,6 +323,10 @@ func (s *SimpleStreams) parseIndex() (*SimpleStreamsIndex, error) {
 	}
 	defer r.Body.Close()
 
+	if r.StatusCode != http.StatusOK {
+		return nil, fmt.Errorf("Unable to fetch %s: %s", url, r.Status)
+	}
+
 	body, err := ioutil.ReadAll(r.Body)
 	if err != nil {
 		return nil, err
@@ -324,7 +349,8 @@ func (s *SimpleStreams) parseManifest(path string) (*SimpleStreamsManifest, erro
 		return s.cachedManifest[path], nil
 	}
 
-	req, err := http.NewRequest("GET", fmt.Sprintf("%s/%s", s.url, path), nil)
+	url := fmt.Sprintf("%s/%s", s.url, path)
+	req, err := http.NewRequest("GET", url, nil)
 	if err != nil {
 		return nil, err
 	}
@@ -339,6 +365,10 @@ func (s *SimpleStreams) parseManifest(path string) (*SimpleStreamsManifest, erro
 	}
 	defer r.Body.Close()
 
+	if r.StatusCode != http.StatusOK {
+		return nil, fmt.Errorf("Unable to fetch %s: %s", url, r.Status)
+	}
+
 	body, err := ioutil.ReadAll(r.Body)
 	if err != nil {
 		return nil, err
@@ -356,8 +386,8 @@ func (s *SimpleStreams) parseManifest(path string) (*SimpleStreamsManifest, erro
 	return &ssManifest, nil
 }
 
-func (s *SimpleStreams) applyAliases(images []shared.ImageInfo) ([]shared.ImageInfo, map[string]*shared.ImageAliasesEntry, error) {
-	aliases := map[string]*shared.ImageAliasesEntry{}
+func (s *SimpleStreams) applyAliases(images []api.Image) ([]api.Image, map[string]*api.ImageAliasesEntry, error) {
+	aliases := map[string]*api.ImageAliasesEntry{}
 
 	sort.Sort(ssSortImage(images))
 
@@ -369,7 +399,7 @@ func (s *SimpleStreams) applyAliases(images []shared.ImageInfo) ([]shared.ImageI
 		}
 	}
 
-	addAlias := func(name string, fingerprint string) *shared.ImageAlias {
+	addAlias := func(name string, fingerprint string) *api.ImageAlias {
 		if defaultOS != "" {
 			name = strings.TrimPrefix(name, fmt.Sprintf("%s/", defaultOS))
 		}
@@ -378,17 +408,17 @@ func (s *SimpleStreams) applyAliases(images []shared.ImageInfo) ([]shared.ImageI
 			return nil
 		}
 
-		alias := shared.ImageAliasesEntry{}
+		alias := api.ImageAliasesEntry{}
 		alias.Name = name
 		alias.Target = fingerprint
 		aliases[name] = &alias
 
-		return &shared.ImageAlias{Name: name}
+		return &api.ImageAlias{Name: name}
 	}
 
 	architectureName, _ := osarch.ArchitectureGetLocal()
 
-	newImages := []shared.ImageInfo{}
+	newImages := []api.Image{}
 	for _, image := range images {
 		if image.Aliases != nil {
 			// Build a new list of aliases from the provided ones
@@ -418,12 +448,12 @@ func (s *SimpleStreams) applyAliases(images []shared.ImageInfo) ([]shared.ImageI
 	return newImages, aliases, nil
 }
 
-func (s *SimpleStreams) getImages() ([]shared.ImageInfo, map[string]*shared.ImageAliasesEntry, error) {
+func (s *SimpleStreams) getImages() ([]api.Image, map[string]*api.ImageAliasesEntry, error) {
 	if s.cachedImages != nil && s.cachedAliases != nil {
 		return s.cachedImages, s.cachedAliases, nil
 	}
 
-	images := []shared.ImageInfo{}
+	images := []api.Image{}
 
 	// Load the main index
 	ssIndex, err := s.parseIndex()
@@ -467,7 +497,7 @@ func (s *SimpleStreams) getImages() ([]shared.ImageInfo, map[string]*shared.Imag
 	return images, aliases, nil
 }
 
-func (s *SimpleStreams) getPaths(fingerprint string) ([][]string, error) {
+func (s *SimpleStreams) GetFiles(fingerprint string) (map[string]SimpleStreamsFile, error) {
 	// Load the main index
 	ssIndex, err := s.parseIndex()
 	if err != nil {
@@ -495,11 +525,21 @@ func (s *SimpleStreams) getPaths(fingerprint string) ([][]string, error) {
 
 		for _, image := range manifestImages {
 			if strings.HasPrefix(image.Fingerprint, fingerprint) {
-				urls := [][]string{}
+				files := map[string]SimpleStreamsFile{}
+
 				for _, path := range downloads[image.Fingerprint] {
-					urls = append(urls, []string{path[0], path[1], path[2]})
+					size, err := strconv.ParseInt(path[3], 10, 64)
+					if err != nil {
+						return nil, err
+					}
+
+					files[path[2]] = SimpleStreamsFile{
+						Path:   path[0],
+						Sha256: path[1],
+						Size:   size}
 				}
-				return urls, nil
+
+				return files, nil
 			}
 		}
 	}
@@ -524,20 +564,20 @@ func (s *SimpleStreams) downloadFile(path string, hash string, target string, pr
 			req.Header.Set("User-Agent", s.useragent)
 		}
 
-		resp, err := s.http.Do(req)
+		r, err := s.http.Do(req)
 		if err != nil {
 			return err
 		}
-		defer resp.Body.Close()
+		defer r.Body.Close()
 
-		if resp.StatusCode != http.StatusOK {
-			return fmt.Errorf("invalid simplestreams source: got %d looking for %s", resp.StatusCode, path)
+		if r.StatusCode != http.StatusOK {
+			return fmt.Errorf("Unable to fetch %s: %s", url, r.Status)
 		}
 
 		body := &ioprogress.ProgressReader{
-			ReadCloser: resp.Body,
+			ReadCloser: r.Body,
 			Tracker: &ioprogress.ProgressTracker{
-				Length:  resp.ContentLength,
+				Length:  r.ContentLength,
 				Handler: progress,
 			},
 		}
@@ -573,13 +613,13 @@ func (s *SimpleStreams) downloadFile(path string, hash string, target string, pr
 	return nil
 }
 
-func (s *SimpleStreams) ListAliases() (shared.ImageAliases, error) {
+func (s *SimpleStreams) ListAliases() ([]api.ImageAliasesEntry, error) {
 	_, aliasesMap, err := s.getImages()
 	if err != nil {
 		return nil, err
 	}
 
-	aliases := shared.ImageAliases{}
+	aliases := []api.ImageAliasesEntry{}
 
 	for _, alias := range aliasesMap {
 		aliases = append(aliases, *alias)
@@ -588,38 +628,46 @@ func (s *SimpleStreams) ListAliases() (shared.ImageAliases, error) {
 	return aliases, nil
 }
 
-func (s *SimpleStreams) ListImages() ([]shared.ImageInfo, error) {
+func (s *SimpleStreams) ListImages() ([]api.Image, error) {
 	images, _, err := s.getImages()
 	return images, err
 }
 
-func (s *SimpleStreams) GetAlias(name string) string {
+func (s *SimpleStreams) GetAlias(name string) (*api.ImageAliasesEntry, error) {
 	_, aliasesMap, err := s.getImages()
 	if err != nil {
-		return ""
+		return nil, err
 	}
 
 	alias, ok := aliasesMap[name]
 	if !ok {
-		return ""
+		return nil, fmt.Errorf("Alias '%s' doesn't exist", name)
 	}
 
-	return alias.Target
+	return alias, nil
 }
 
-func (s *SimpleStreams) GetImageInfo(fingerprint string) (*shared.ImageInfo, error) {
+func (s *SimpleStreams) GetImage(fingerprint string) (*api.Image, error) {
 	images, _, err := s.getImages()
 	if err != nil {
 		return nil, err
 	}
 
+	matches := []api.Image{}
+
 	for _, image := range images {
 		if strings.HasPrefix(image.Fingerprint, fingerprint) {
-			return &image, nil
+			matches = append(matches, image)
 		}
 	}
 
-	return nil, fmt.Errorf("The requested image couldn't be found.")
+	if len(matches) == 0 {
+		return nil, fmt.Errorf("The requested image couldn't be found.")
+	} else if len(matches) > 1 {
+		return nil, fmt.Errorf("More than one match for the provided partial fingerprint.")
+	}
+
+	return &matches[0], nil
 }
 
 func (s *SimpleStreams) ExportImage(image string, target string) (string, error) {
@@ -627,16 +675,16 @@ func (s *SimpleStreams) ExportImage(image string, target string) (string, error)
 		return "", fmt.Errorf("Split images can only be written to a directory.")
 	}
 
-	paths, err := s.getPaths(image)
+	files, err := s.GetFiles(image)
 	if err != nil {
 		return "", err
 	}
 
-	for _, path := range paths {
-		fields := strings.Split(path[0], "/")
+	for _, file := range files {
+		fields := strings.Split(file.Path, "/")
 		targetFile := filepath.Join(target, fields[len(fields)-1])
 
-		err := s.downloadFile(path[0], path[1], targetFile, nil)
+		err := s.downloadFile(file.Path, file.Sha256, targetFile, nil)
 		if err != nil {
 			return "", err
 		}
@@ -645,18 +693,15 @@ func (s *SimpleStreams) ExportImage(image string, target string) (string, error)
 	return target, nil
 }
 
-func (s *SimpleStreams) Download(image string, file string, target string, progress func(int64, int64)) error {
-	paths, err := s.getPaths(image)
+func (s *SimpleStreams) Download(image string, fileType string, target string, progress func(int64, int64)) error {
+	files, err := s.GetFiles(image)
 	if err != nil {
 		return err
 	}
 
-	for _, path := range paths {
-		if file != path[2] {
-			continue
-		}
-
-		return s.downloadFile(path[0], path[1], target, progress)
+	file, ok := files[fileType]
+	if ok {
+		return s.downloadFile(file.Path, file.Sha256, target, progress)
 	}
 
 	return fmt.Errorf("The file couldn't be found.")
