@@ -9,6 +9,7 @@ import (
 	"io"
 	"os"
 	"os/exec"
+	"path/filepath"
 	"strings"
 	"sync"
 	"sync/atomic"
@@ -19,18 +20,19 @@ import (
 // #cgo LDFLAGS: -lutil -lpthread
 /*
 #define _GNU_SOURCE
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <grp.h>
-#include <pty.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <grp.h>
 #include <limits.h>
 #include <poll.h>
-#include <string.h>
+#include <pty.h>
+#include <pwd.h>
 #include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <unistd.h>
 
 #ifndef AT_SYMLINK_FOLLOW
 #define AT_SYMLINK_FOLLOW    0x400
@@ -90,7 +92,7 @@ void configure_pty(int fd) {
 	return;
 }
 
-void create_pty(int *master, int *slave, int uid, int gid) {
+void create_pty(int *master, int *slave, uid_t uid, gid_t gid) {
 	if (openpty(master, slave, NULL, NULL, NULL) < 0) {
 		fprintf(stderr, "Failed to openpty: %s\n", strerror(errno));
 		return;
@@ -243,11 +245,11 @@ func ShiftOwner(basepath string, path string, uid int, gid int) error {
 	return nil
 }
 
-func OpenPty(uid, gid int) (master *os.File, slave *os.File, err error) {
+func OpenPty(uid, gid int64) (master *os.File, slave *os.File, err error) {
 	fd_master := C.int(-1)
 	fd_slave := C.int(-1)
-	rootUid := C.int(uid)
-	rootGid := C.int(gid)
+	rootUid := C.uid_t(uid)
+	rootGid := C.gid_t(gid)
 
 	C.create_pty(&fd_master, &fd_slave, rootUid, rootGid)
 
@@ -277,36 +279,36 @@ func Pipe() (master *os.File, slave *os.File, err error) {
 	return master, slave, nil
 }
 
-// GroupName is an adaption from https://codereview.appspot.com/4589049.
-func GroupName(gid int) (string, error) {
-	var grp C.struct_group
-	var result *C.struct_group
+// UserId is an adaption from https://codereview.appspot.com/4589049.
+func UserId(name string) (int, error) {
+	var pw C.struct_passwd
+	var result *C.struct_passwd
 
-	bufSize := C.size_t(C.sysconf(C._SC_GETGR_R_SIZE_MAX))
+	bufSize := C.size_t(C.sysconf(C._SC_GETPW_R_SIZE_MAX))
 	buf := C.malloc(bufSize)
 	if buf == nil {
-		return "", fmt.Errorf("allocation failed")
+		return -1, fmt.Errorf("allocation failed")
 	}
 	defer C.free(buf)
 
-	// mygetgrgid_r is a wrapper around getgrgid_r to
-	// to avoid using gid_t because C.gid_t(gid) for
-	// unknown reasons doesn't work on linux.
-	rv := C.mygetgrgid_r(C.int(gid),
-		&grp,
+	cname := C.CString(name)
+	defer C.free(unsafe.Pointer(cname))
+
+	rv := C.getpwnam_r(cname,
+		&pw,
 		(*C.char)(buf),
 		bufSize,
 		&result)
 
 	if rv != 0 {
-		return "", fmt.Errorf("failed group lookup: %s", syscall.Errno(rv))
+		return -1, fmt.Errorf("failed user lookup: %s", syscall.Errno(rv))
 	}
 
 	if result == nil {
-		return "", fmt.Errorf("unknown group %d", gid)
+		return -1, fmt.Errorf("unknown user %s", name)
 	}
 
-	return C.GoString(result.gr_name), nil
+	return int(C.int(result.pw_uid)), nil
 }
 
 // GroupId is an adaption from https://codereview.appspot.com/4589049.
@@ -321,9 +323,6 @@ func GroupId(name string) (int, error) {
 	}
 	defer C.free(buf)
 
-	// mygetgrgid_r is a wrapper around getgrgid_r to
-	// to avoid using gid_t because C.gid_t(gid) for
-	// unknown reasons doesn't work on linux.
 	cname := C.CString(name)
 	defer C.free(unsafe.Pointer(cname))
 
@@ -370,7 +369,7 @@ func GetFileStat(p string) (uid int, gid int, major int, minor int,
 func IsMountPoint(name string) bool {
 	_, err := exec.LookPath("mountpoint")
 	if err == nil {
-		err = exec.Command("mountpoint", "-q", name).Run()
+		_, err = RunCommand("mountpoint", "-q", name)
 		if err != nil {
 			return false
 		}
@@ -413,7 +412,7 @@ func ReadLastNLines(f *os.File, lines int) (string, error) {
 		}
 
 		if lines < 0 {
-			return string(data[i+1 : len(data)]), nil
+			return string(data[i+1:]), nil
 		}
 	}
 
@@ -545,7 +544,6 @@ func ExecReaderToChannel(r io.Reader, bufferSize int, exited <-chan bool, fd int
 		close(ch)
 	}
 
-	// COMMENT(brauner):
 	// [1]: This function has just one job: Dealing with the case where we
 	// are running an interactive shell session where we put a process in
 	// the background that does hold stdin/stdout open, but does not
@@ -592,14 +590,12 @@ func ExecReaderToChannel(r io.Reader, bufferSize int, exited <-chan bool, fd int
 
 			ret, revents, err := GetPollRevents(fd, -1, (POLLIN | POLLPRI | POLLERR | POLLHUP | POLLRDHUP))
 			if ret < 0 {
-				// COMMENT(brauner):
 				// This condition is only reached in cases where we are massively f*cked since we even handle
 				// EINTR in the underlying C wrapper around poll(). So let's exit here.
 				LogErrorf("Failed to poll(POLLIN | POLLPRI | POLLERR | POLLHUP | POLLRDHUP) on file descriptor: %s. Exiting.", err)
 				return
 			}
 
-			// COMMENT(brauner):
 			// [2]: If the process exits before all its data has been read by us and no other process holds stdin or
 			// stdout open, then we will observe a (POLLHUP | POLLRDHUP | POLLIN) event. This means, we need to
 			// keep on reading from the pty file descriptor until we get a simple POLLHUP back.
@@ -616,7 +612,6 @@ func ExecReaderToChannel(r io.Reader, bufferSize int, exited <-chan bool, fd int
 			}
 
 			if ((revents & (POLLIN | POLLPRI)) > 0) && !both {
-				// COMMENT(brauner):
 				// This might appear unintuitive at first but is actually a nice trick: Assume we are running
 				// a shell session in a container and put a process in the background that is writing to
 				// stdout. Now assume the attached process (aka the shell in this example) exits because we
@@ -634,7 +629,6 @@ func ExecReaderToChannel(r io.Reader, bufferSize int, exited <-chan bool, fd int
 				// handled and triggers another codepath. (See [2].)"
 				if avoidAtomicLoad || atomic.LoadInt32(&attachedChildIsDead) == 1 {
 					avoidAtomicLoad = true
-					// COMMENT(brauner):
 					// Handle race between atomic.StorInt32() in the go routine
 					// explained in [1] and atomic.LoadInt32() in the go routine
 					// here:
@@ -642,7 +636,7 @@ func ExecReaderToChannel(r io.Reader, bufferSize int, exited <-chan bool, fd int
 					// still be handling a pure POLLIN event from a write prior to the childs
 					// exit. But the child might have exited right before and performed
 					// atomic.StoreInt32() to update attachedChildIsDead before we
-					// performed our atomic.LoadInt32(). This means we accidently hit this
+					// performed our atomic.LoadInt32(). This means we accidentally hit this
 					// codepath and are misinformed about the available poll() events. So we
 					// need to perform a non-blocking poll() again to exclude that case:
 					//
@@ -683,7 +677,6 @@ func ExecReaderToChannel(r io.Reader, bufferSize int, exited <-chan bool, fd int
 				nr, err = r.Read(read)
 			}
 
-			// COMMENT(brauner):
 			// The attached process has exited and we have read all data that may have
 			// been buffered.
 			if ((revents & (POLLHUP | POLLRDHUP)) > 0) && !both {
@@ -702,3 +695,84 @@ func ExecReaderToChannel(r io.Reader, bufferSize int, exited <-chan bool, fd int
 
 	return ch
 }
+
+var ObjectFound = fmt.Errorf("Found requested object.")
+
+func LookupUUIDByBlockDevPath(diskDevice string) (string, error) {
+	uuid := ""
+	readUUID := func(path string, info os.FileInfo, err error) error {
+		if err != nil {
+			return err
+		}
+
+		if (info.Mode() & os.ModeSymlink) == os.ModeSymlink {
+			link, err := os.Readlink(path)
+			if err != nil {
+				return err
+			}
+
+			// filepath.Join() will call Clean() on the result and
+			// thus resolve those ugly "../../" parts that make it
+			// hard to compare the strings.
+			absPath := filepath.Join("/dev/disk/by-uuid", link)
+			if absPath == diskDevice {
+				uuid = path
+				// Will allows us to avoid needlessly travers
+				// the whole directory.
+				return ObjectFound
+			}
+		}
+		return nil
+	}
+
+	err := filepath.Walk("/dev/disk/by-uuid", readUUID)
+	if err != nil && err != ObjectFound {
+		return "", fmt.Errorf("Failed to detect UUID: %s.", err)
+	}
+
+	if uuid == "" {
+		return "", fmt.Errorf("Failed to detect UUID.")
+	}
+
+	lastSlash := strings.LastIndex(uuid, "/")
+	return uuid[lastSlash+1:], nil
+}
+
+func LookupBlockDevByUUID(uuid string) (string, error) {
+	detectedPath := ""
+	readPath := func(path string, info os.FileInfo, err error) error {
+		if err != nil {
+			return err
+		}
+
+		if (info.Mode() & os.ModeSymlink) == os.ModeSymlink {
+			link, err := os.Readlink(path)
+			if err != nil {
+				return err
+			}
+
+			if info.Name() == uuid {
+				// filepath.Join() will call Clean() on the
+				// result and thus resolve those ugly "../../"
+				// parts that make it hard to compare the
+				// strings.
+				detectedPath = filepath.Join("/dev/disk/by-uuid", link)
+				// Will allows us to avoid needlessly travers
+				// the whole directory.
+				return ObjectFound
+			}
+		}
+		return nil
+	}
+
+	err := filepath.Walk("/dev/disk/by-uuid", readPath)
+	if err != nil && err != ObjectFound {
+		return "", fmt.Errorf("Failed to detect disk device: %s.", err)
+	}
+
+	if detectedPath == "" {
+		return "", fmt.Errorf("Failed to detect disk device.")
+	}
+
+	return detectedPath, nil
+}
