@@ -68,6 +68,9 @@ spawn_lxd() {
   lxddir=${1}
   shift
 
+  storage=${1}
+  shift
+
   # Copy pre generated Certs
   cp deps/server.crt "${lxddir}"
   cp deps/server.key "${lxddir}"
@@ -109,8 +112,10 @@ spawn_lxd() {
     LXD_DIR="${lxddir}" lxc network attach-profile lxdbr0 default eth0
   fi
 
-  echo "==> Configuring storage backend"
-  "$LXD_BACKEND"_configure "${lxddir}"
+  if [ "${storage}" = true ]; then
+    echo "==> Configuring storage backend"
+    "$LXD_BACKEND"_configure "${lxddir}"
+  fi
 }
 
 lxc() {
@@ -235,12 +240,23 @@ kill_lxd() {
       lxc image delete "${image}" --force-local || true
     done
 
+    # Delete all networks
+    echo "==> Deleting all networks"
+    for network in $(lxc network list --force-local | grep YES | grep "^| " | cut -d' ' -f2); do
+      lxc network delete "${network}" --force-local || true
+    done
+
     # Delete all profiles
     echo "==> Deleting all profiles"
     for profile in $(lxc profile list --force-local | tail -n+3 | grep "^| " | cut -d' ' -f2); do
       lxc profile delete "${profile}" --force-local || true
     done
 
+    echo "==> Deleting all storage pools"
+    for storage in $(lxc storage list --force-local | tail -n+3 | grep "^| " | cut -d' ' -f2); do
+      lxc storage delete "${storage}" --force-local || true
+    done
+
     echo "==> Checking for locked DB tables"
     for table in $(echo .tables | sqlite3 "${daemon_dir}/lxd.db"); do
       echo "SELECT * FROM ${table};" | sqlite3 "${daemon_dir}/lxd.db" >/dev/null
@@ -251,6 +267,7 @@ kill_lxd() {
 
     # Cleanup shmounts (needed due to the forceful kill)
     find "${daemon_dir}" -name shmounts -exec "umount" "-l" "{}" \; >/dev/null 2>&1 || true
+    find "${daemon_dir}" -name devlxd -exec "umount" "-l" "{}" \; >/dev/null 2>&1 || true
 
     check_leftovers="true"
   fi
@@ -283,6 +300,8 @@ kill_lxd() {
     check_empty_table "${daemon_dir}/lxd.db" "containers_devices"
     check_empty_table "${daemon_dir}/lxd.db" "containers_devices_config"
     check_empty_table "${daemon_dir}/lxd.db" "containers_profiles"
+    check_empty_table "${daemon_dir}/lxd.db" "networks"
+    check_empty_table "${daemon_dir}/lxd.db" "networks_config"
     check_empty_table "${daemon_dir}/lxd.db" "images"
     check_empty_table "${daemon_dir}/lxd.db" "images_aliases"
     check_empty_table "${daemon_dir}/lxd.db" "images_properties"
@@ -291,6 +310,10 @@ kill_lxd() {
     check_empty_table "${daemon_dir}/lxd.db" "profiles_config"
     check_empty_table "${daemon_dir}/lxd.db" "profiles_devices"
     check_empty_table "${daemon_dir}/lxd.db" "profiles_devices_config"
+    check_empty_table "${daemon_dir}/lxd.db" "storage_pools"
+    check_empty_table "${daemon_dir}/lxd.db" "storage_pools_config"
+    check_empty_table "${daemon_dir}/lxd.db" "storage_volumes"
+    check_empty_table "${daemon_dir}/lxd.db" "storage_volumes_config"
   fi
 
   # teardown storage
@@ -360,7 +383,7 @@ wipe() {
 
   # shellcheck disable=SC2009
   ps aux | grep lxc-monitord | grep "${1}" | awk '{print $2}' | while read -r pid; do
-    kill -9 "${pid}"
+    kill -9 "${pid}" || true
   done
 
   if [ -f "${TEST_DIR}/loops" ]; then
@@ -375,6 +398,60 @@ wipe() {
   rm -Rf "${1}"
 }
 
+configure_loop_device() {
+  lv_loop_file=$(mktemp -p "${TEST_DIR}" XXXX.img)
+  truncate -s 10G "${lv_loop_file}"
+  pvloopdev=$(losetup --show -f "${lv_loop_file}")
+  if [ ! -e "${pvloopdev}" ]; then
+    echo "failed to setup loop"
+    false
+  fi
+  echo "${pvloopdev}" >> "${TEST_DIR}/loops"
+
+  # The following code enables to return a value from a shell function by
+  # calling the function as: fun VAR1
+
+  # shellcheck disable=2039
+  local  __tmp1="${1}"
+  # shellcheck disable=2039
+  local  res1="${lv_loop_file}"
+  if [ "${__tmp1}" ]; then
+      eval "${__tmp1}='${res1}'"
+  fi
+
+  # shellcheck disable=2039
+  local  __tmp2="${2}"
+  # shellcheck disable=2039
+  local  res2="${pvloopdev}"
+  if [ "${__tmp2}" ]; then
+      eval "${__tmp2}='${res2}'"
+  fi
+}
+
+deconfigure_loop_device() {
+  lv_loop_file="${1}"
+  loopdev="${2}"
+
+  SUCCESS=0
+  # shellcheck disable=SC2034
+  for i in $(seq 10); do
+    if losetup -d "${loopdev}"; then
+      SUCCESS=1
+      break
+    fi
+
+    sleep 0.5
+  done
+
+  if [ "${SUCCESS}" = "0" ]; then
+    echo "Failed to tear down loop device"
+    false
+  fi
+
+  rm -f "${lv_loop_file}"
+  sed -i "\|^${loopdev}|d" "${TEST_DIR}/loops"
+}
+
 # Must be set before cleanup()
 TEST_CURRENT=setup
 TEST_RESULT=failure
@@ -402,14 +479,14 @@ export LXD_CONF
 LXD_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
 export LXD_DIR
 chmod +x "${LXD_DIR}"
-spawn_lxd "${LXD_DIR}"
+spawn_lxd "${LXD_DIR}" true
 LXD_ADDR=$(cat "${LXD_DIR}/lxd.addr")
 export LXD_ADDR
 
 # Setup the second LXD
 LXD2_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
 chmod +x "${LXD2_DIR}"
-spawn_lxd "${LXD2_DIR}"
+spawn_lxd "${LXD2_DIR}" true
 LXD2_ADDR=$(cat "${LXD2_DIR}/lxd.addr")
 export LXD2_ADDR
 
@@ -433,7 +510,7 @@ fi
 run_test test_check_deps "checking dependencies"
 run_test test_static_analysis "static analysis"
 run_test test_database_update "database schema updates"
-run_test test_remote_url "remote  url handling"
+run_test test_remote_url "remote url handling"
 run_test test_remote_admin "remote administration"
 run_test test_remote_usage "remote usage"
 run_test test_basic_usage "basic usage"
@@ -456,5 +533,9 @@ run_test test_migration "migration"
 run_test test_fdleak "fd leak"
 run_test test_cpu_profiling "CPU profiling"
 run_test test_mem_profiling "memory profiling"
+run_test test_storage "storage"
+run_test test_lxd_autoinit "lxd init auto"
+run_test test_storage_profiles "storage profiles"
+run_test test_container_import "container import"
 
 TEST_RESULT=success
