@@ -1,67 +1,65 @@
+#!/bin/sh
+
 test_snapshots() {
   ensure_import_testimage
   ensure_has_localhost_remote "${LXD_ADDR}"
 
-  # shellcheck disable=2039
-  local lxd_backend
-  lxd_backend=$(storage_backend "$LXD_DIR")
-
   lxc init testimage foo
 
   lxc snapshot foo
   # FIXME: make this backend agnostic
-  if [ "${lxd_backend}" = "dir" ]; then
+  if [ "${LXD_BACKEND}" = "dir" ]; then
     [ -d "${LXD_DIR}/snapshots/foo/snap0" ]
   fi
 
   lxc snapshot foo
   # FIXME: make this backend agnostic
-  if [ "${lxd_backend}" = "dir" ]; then
+  if [ "${LXD_BACKEND}" = "dir" ]; then
     [ -d "${LXD_DIR}/snapshots/foo/snap1" ]
   fi
 
   lxc snapshot foo tester
   # FIXME: make this backend agnostic
-  if [ "${lxd_backend}" = "dir" ]; then
+  if [ "${LXD_BACKEND}" = "dir" ]; then
     [ -d "${LXD_DIR}/snapshots/foo/tester" ]
   fi
 
   lxc copy foo/tester foosnap1
   # FIXME: make this backend agnostic
-  if [ "${lxd_backend}" != "lvm" ]; then
+  if [ "${LXD_BACKEND}" != "lvm" ]; then
     [ -d "${LXD_DIR}/containers/foosnap1/rootfs" ]
   fi
 
   lxc delete foo/snap0
   # FIXME: make this backend agnostic
-  if [ "${lxd_backend}" = "dir" ]; then
+  if [ "${LXD_BACKEND}" = "dir" ]; then
     [ ! -d "${LXD_DIR}/snapshots/foo/snap0" ]
   fi
 
   # no CLI for this, so we use the API directly (rename a snapshot)
   wait_for "${LXD_ADDR}" my_curl -X POST "https://${LXD_ADDR}/1.0/containers/foo/snapshots/tester" -d "{\"name\":\"tester2\"}"
   # FIXME: make this backend agnostic
-  if [ "${lxd_backend}" = "dir" ]; then
+  if [ "${LXD_BACKEND}" = "dir" ]; then
     [ ! -d "${LXD_DIR}/snapshots/foo/tester" ]
   fi
 
   lxc move foo/tester2 foo/tester-two
   lxc delete foo/tester-two
   # FIXME: make this backend agnostic
-  if [ "${lxd_backend}" = "dir" ]; then
+  if [ "${LXD_BACKEND}" = "dir" ]; then
     [ ! -d "${LXD_DIR}/snapshots/foo/tester-two" ]
   fi
 
   lxc snapshot foo namechange
   # FIXME: make this backend agnostic
-  if [ "${lxd_backend}" = "dir" ]; then
+  if [ "${LXD_BACKEND}" = "dir" ]; then
     [ -d "${LXD_DIR}/snapshots/foo/namechange" ]
   fi
   lxc move foo foople
   [ ! -d "${LXD_DIR}/containers/foo" ]
   [ -d "${LXD_DIR}/containers/foople" ]
   # FIXME: make this backend agnostic
-  if [ "${lxd_backend}" = "dir" ]; then
+  if [ "${LXD_BACKEND}" = "dir" ]; then
     [ -d "${LXD_DIR}/snapshots/foople/namechange" ]
     [ -d "${LXD_DIR}/snapshots/foople/namechange" ]
   fi
@@ -73,10 +71,6 @@ test_snapshots() {
 }
 
 test_snap_restore() {
-  # shellcheck disable=2039
-  local lxd_backend
-  lxd_backend=$(storage_backend "$LXD_DIR")
-
   ensure_import_testimage
   ensure_has_localhost_remote "${LXD_ADDR}"
 
@@ -119,7 +113,7 @@ test_snap_restore() {
 
   ##########################################################
 
-  if [ "${lxd_backend}" != "zfs" ]; then
+  if [ "${LXD_BACKEND}" != "zfs" ]; then
     # The problem here is that you can't `zfs rollback` to a snapshot with a
     # parent, which snap0 has (snap1).
     restore_and_compare_fs snap0
@@ -149,7 +143,7 @@ test_snap_restore() {
   # Start container and then restore snapshot to verify the running state after restore.
   lxc start bar
 
-  if [ "${lxd_backend}" != "zfs" ]; then
+  if [ "${LXD_BACKEND}" != "zfs" ]; then
     # see comment above about snap0
     restore_and_compare_fs snap0
 
@@ -160,6 +154,18 @@ test_snap_restore() {
   lxc stop --force bar
 
   lxc delete bar
+
+  # Test if container's with hyphen's in their names are treated correctly.
+  if [ "${LXD_BACKEND}" = "lvm" ]; then
+	  lxc launch testimage a-b
+	  lxc snapshot a-b base
+	  lxc restore a-b base
+
+	  lxc snapshot a-b c-d
+	  lxc restore a-b c-d
+
+	  lxc delete -f a-b
+  fi
 }
 
 restore_and_compare_fs() {
@@ -169,7 +175,7 @@ restore_and_compare_fs() {
   lxc restore bar "${snap}"
 
   # FIXME: make this backend agnostic
-  if [ "$(storage_backend "$LXD_DIR")" = "dir" ]; then
+  if [ "${LXD_BACKEND}" = "dir" ]; then
     # Recursive diff of container FS
     diff -r "${LXD_DIR}/containers/bar/rootfs" "${LXD_DIR}/snapshots/bar/${snap}/rootfs"
   fi
