@@ -1,3 +1,5 @@
+#!/bin/sh
+
 safe_pot_hash() {
   sed -e "/Project-Id-Version/,/Content-Transfer-Encoding/d" -e "/^#/d" "po/lxd.pot" | md5sum | cut -f1 -d" "
 }
@@ -8,18 +10,11 @@ test_static_analysis() {
 
     cd ../
     # Python3 static analysis
-    if which flake8 >/dev/null 2>&1; then
-      flake8 test/deps/import-busybox scripts/lxd-setup-lvm-storage
-    else
-      echo "flake8 not found, python static analysis disabled"
-    fi
+    pep8 test/deps/import-busybox
+    pyflakes3 test/deps/import-busybox
 
     # Shell static analysis
-    if which shellcheck >/dev/null 2>&1; then
-      shellcheck --shell sh test/*.sh test/includes/*.sh test/suites/*.sh test/backends/*.sh
-    else
-      echo "shellcheck not found, shell static analysis disabled"
-    fi
+    shellcheck test/main.sh test/suites/* test/backends/*
 
     # Go static analysis
     ## Functions starting by empty line
@@ -41,18 +36,7 @@ test_static_analysis() {
 
     ## golint
     if which golint >/dev/null 2>&1; then
-      golint -set_exit_status client/
-      golint -set_exit_status lxc/config/
       golint -set_exit_status shared/api/
-      golint -set_exit_status shared/cmd/
-      golint -set_exit_status shared/gnuflag/
-      golint -set_exit_status shared/i18n/
-      golint -set_exit_status shared/ioprogress/
-      golint -set_exit_status shared/logger/
-      golint -set_exit_status shared/logging/
-      golint -set_exit_status shared/termios/
-      golint -set_exit_status shared/version/
-      golint -set_exit_status test/deps/
     fi
 
     ## deadcode
@@ -65,11 +49,10 @@ test_static_analysis() {
     fi
 
     if which godeps >/dev/null 2>&1; then
-      OUT=$(godeps -T ./client ./lxc/config ./shared/api 2>/dev/null | cut -f1 | diff -u test/godeps.list - || true)
-      if [ -n "${OUT}" ]; then
+      OUT=$(godeps . ./shared | cut -f1)
+      if [ "${OUT}" != "$(printf "github.com/gorilla/websocket\ngopkg.in/yaml.v2\n")" ]; then
         echo "ERROR: you added a new dependency to the client or shared; please make sure this is what you want"
         echo "${OUT}"
-        exit 1
       fi
     fi
 
